@@ -27,6 +27,7 @@
 #    include <stdlib.h>
 #    include <stdio.h>
 #    include <string.h>
+#    include <ctype.h>
 #    ifndef __BEOS__
 #        include <errno.h>
 #    else
@@ -120,13 +121,13 @@ typedef signed __int64 int64_t;
 
 #include <inttypes.h>
 
-#ifdef HAVE_AV_CONFIG_H
-
 #ifndef int64_t_C
 #define int64_t_C(c)     (c ## LL)
 #define uint64_t_C(c)    (c ## ULL)
 #endif
 
+#ifdef HAVE_AV_CONFIG_H
+
 #ifdef USE_FASTMEMCPY
 #include "fastmemcpy.h"
 #endif
@@ -140,13 +141,14 @@ typedef signed __int64 int64_t;
 
 /* unix */
 
-#    include <inttypes.h>
+#include <inttypes.h>
 
-#    ifdef HAVE_AV_CONFIG_H
-#        ifndef int64_t_C
-#            define int64_t_C(c)     (c ## LL)
-#            define uint64_t_C(c)    (c ## ULL)
-#        endif
+#ifndef int64_t_C
+#define int64_t_C(c)     (c ## LL)
+#define uint64_t_C(c)    (c ## ULL)
+#endif
+
+#ifdef HAVE_AV_CONFIG_H
 
 #        ifdef USE_FASTMEMCPY
 #            include "fastmemcpy.h"
@@ -1047,7 +1049,7 @@ static inline int ff_sqrt(int a)
  */
 static inline int ff_get_fourcc(const char *s){
     assert( strlen(s)==4 );
-    
+
     return (s[0]) + (s[1]<<8) + (s[2]<<16) + (s[3]<<24);
 }
 
