@@ -17,21 +17,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include "avformat.h"
-#include <ctype.h>
-#ifdef CONFIG_WIN32
-#define strcasecmp _stricmp
-#include <sys/types.h>
-#include <sys/timeb.h>
-#elif defined(CONFIG_OS2)
-#include <string.h>
-#define strcasecmp stricmp
-#include <sys/time.h>
-#else
-#include <unistd.h>
-#include <fcntl.h>
-#include <sys/time.h>
-#endif
-#include <time.h>
 
 AVInputFormat *first_iformat;
 AVOutputFormat *first_oformat;
@@ -315,6 +300,9 @@ int av_open_input_file(AVFormatContext **ic_ptr, const char *filename,
             goto fail;
         }
     }
+
+    ic->duration = AV_NOPTS_VALUE;
+    ic->start_time = AV_NOPTS_VALUE;
     pstrcpy(ic->filename, sizeof(ic->filename), filename);
     pd->filename = ic->filename;
     pd->buf = buf;
@@ -467,7 +455,7 @@ int av_find_stream_info(AVFormatContext *ic)
            in a file */
         min_read_size = 3000000;
     } else {
-        min_read_size = 125000;
+        min_read_size = 250000;
     }
     /* max read size is 2 seconds of video max */
     max_read_size = min_read_size * 5;
@@ -581,7 +569,8 @@ int av_find_stream_info(AVFormatContext *ic)
                        examining this stream */
                     /* XXX: add a codec info so that we can decide if
                        the codec can repeat frames */
-                    if (st->codec.codec_id == CODEC_ID_MPEG1VIDEO && 
+                    if ((st->codec.codec_id == CODEC_ID_MPEG1VIDEO ||
+                         st->codec.codec_id == CODEC_ID_MPEG2VIDEO) &&
                         ic->iformat != &mpegts_demux &&
                         st->codec.sub_id == 2) {
                         /* for mpeg2 video, we want to know the real
@@ -694,8 +683,14 @@ AVStream *av_new_stream(AVFormatContext *s, int id)
         return NULL;
     avcodec_get_context_defaults(&st->codec);
 
+    if (s->iformat) {
+        /* no default bitrate if decoding */
+        st->codec.bit_rate = 0;
+    }
     st->index = s->nb_streams;
     st->id = id;
+    st->start_time = AV_NOPTS_VALUE;
+    st->duration = AV_NOPTS_VALUE;
     s->streams[s->nb_streams++] = st;
     return st;
 }
@@ -869,8 +864,12 @@ typedef struct {
 } AbvEntry;
 
 static AbvEntry frame_abvs[] = {
-    { "ntsc",      352, 240, 30000, 1001 },
-    { "pal",       352, 288,    25,    1 },
+    { "ntsc",      720, 480, 30000, 1001 },
+    { "pal",       720, 576,    25,    1 },
+    { "qntsc",     352, 240, 30000, 1001 }, /* VCD compliant ntsc */
+    { "qpal",      352, 288,    25,    1 }, /* VCD compliant pal */
+    { "sntsc",     640, 480, 30000, 1001 }, /* square pixel ntsc */
+    { "spal",      768, 576,    25,    1 }, /* square pixel pal */
     { "film",      352, 240,    24,    1 },
     { "ntsc-film", 352, 240, 24000, 1001 },
     { "sqcif",     128,  96,     0,    0 },
@@ -941,38 +940,6 @@ int parse_frame_rate(int *frame_rate, int *frame_rate_base, const char *arg)
         return 0;
 }
 
-int64_t av_gettime(void)
-{
-#ifdef CONFIG_WIN32
-    struct _timeb tb;
-    _ftime(&tb);
-    return ((int64_t)tb.time * int64_t_C(1000) + (int64_t)tb.millitm) * int64_t_C(1000);
-#else
-    struct timeval tv;
-    gettimeofday(&tv,NULL);
-    return (int64_t)tv.tv_sec * 1000000 + tv.tv_usec;
-#endif
-}
-
-static time_t mktimegm(struct tm *tm)
-{
-    time_t t;
-
-    int y = tm->tm_year + 1900, m = tm->tm_mon + 1, d = tm->tm_mday;
-
-    if (m < 3) {
-        m += 12;
-        y--;
-    }
-
-    t = 86400 * 
-        (d + (153 * m - 457) / 5 + 365 * y + y / 4 - y / 100 + y / 400 - 719469);
-
-    t += 3600 * tm->tm_hour + 60 * tm->tm_min + tm->tm_sec;
-
-    return t;
-}
-
 /* syntax: '?tag1=val1&tag2=val2...'. Little URL decoding is done. Return
    1 if found */
 int find_info_tag(char *arg, int arg_size, const char *tag1, const char *info)
@@ -1038,12 +1005,6 @@ int get_frame_filename(char *buf, int buf_size,
                     nd = nd * 10 + *p++ - '0';
                 }
                 c = *p++;
-                if (c == '*' && nd > 0) {
-                    // The nd field is actually the modulus
-                    number = number % nd;
-                    c = *p++;
-                    nd = 0;
-                }
             } while (isdigit(c));
 
             switch(c) {
