@@ -18,9 +18,9 @@
 #include "dialogbox.h"
 #include "mythdialogs.h"
 
-bool print_verbose_messages = false;    
+int print_verbose_messages = VB_IMPORTANT | VB_GENERAL;
 
-MythContext::MythContext(const QString &binversion, bool gui)
+MythContext::MythContext(const QString &binversion, bool gui, bool lcd)
            : QObject()
 {
     qInitNetworkProtocols();
@@ -41,6 +41,7 @@ MythContext::MythContext(const QString &binversion, bool gui)
 
     language = "";
     m_themeloaded = false;
+    m_backgroundimage = NULL;
 
     m_db = QSqlDatabase::addDatabase("QMYSQL3", "MythContext");
 
@@ -59,6 +60,8 @@ MythContext::MythContext(const QString &binversion, bool gui)
         m_localhostname = localhostname;
     }
 
+    m_xbase = m_ybase = 0;
+
     if (gui)
     {
         m_height = QApplication::desktop()->height();
@@ -78,14 +81,16 @@ MythContext::MythContext(const QString &binversion, bool gui)
     expectingReply = false;
 
     mainWindow = NULL;
-	
-    lcd_device = new LCD();
+
+    if (lcd)
+        lcd_device = new LCD();
+    else
+        lcd_device = NULL;
 }
 
 MythContext::~MythContext()
 {
     imageCache.clear();
-    pixmapCache.clear();
 
     if (m_settings)
         delete m_settings;
@@ -96,13 +101,16 @@ MythContext::~MythContext()
         serverSock->close();
         delete serverSock;
     }
+
+    if (lcd_device)
+        delete lcd_device;
 }
 
-void MythContext::ConnectToMasterServer(void)
+bool MythContext::ConnectToMasterServer(void)
 {
     QString server = gContext->GetSetting("MasterServerIP", "localhost");
     int port = gContext->GetNumSetting("MasterServerPort", 6543);
-    ConnectServer(server, port);
+    return ConnectServer(server, port);
 }
 
 bool MythContext::ConnectServer(const QString &hostname, int port)
@@ -127,14 +135,38 @@ bool MythContext::ConnectServer(const QString &hostname, int port)
             cerr << "Connection timed out.\n";
             cerr << "You probably should modify the Master Server settings\n";
             cerr << "in the setup program and set the proper IP address.\n";
-            exit(0);
+            if (m_height && m_width)
+            {
+                MythPopupBox::showOkPopup(mainWindow, "connection failure",
+                                          tr("Connection to the master backend "
+                                             "server timed out.  You probably "
+                                             "should modify the Mater Server "
+                                             "setting in the setup program and "
+                                             "set the proper IP address "
+                                             "there."));
+            }
+            serverSock->close();
+            delete serverSock;
+            serverSock = NULL;
+            return false;
         }
     }
 
     if (serverSock->state() != QSocket::Connected)
     {
         cout << "Could not connect to backend server\n";
-        exit(0);
+        if (m_height && m_width)
+        {
+            MythPopupBox::showOkPopup(mainWindow, "connection failure",
+                                      tr("Could not connect to the master "
+                                         "backend server -- is it running?  Is "
+                                         "the IP address set for it in the "
+                                         "setup program correct?"));
+        }
+        serverSock->close();
+        delete serverSock;
+        serverSock = NULL;
+        return false;
     }
 
     QString str = QString("ANN Playback %1 %2").arg(m_localhostname).arg(true);
@@ -149,11 +181,14 @@ bool MythContext::ConnectServer(const QString &hostname, int port)
 
 QString MythContext::GetMasterHostPrefix(void)
 {
+    QString ret = "";
+
     if (!serverSock)
         ConnectToMasterServer();
 
-    QString ret = QString("myth://%1:%2/").arg(serverSock->peerName())
-                                          .arg(serverSock->peerPort());
+    if (serverSock)
+        ret = QString("myth://%1:%2/").arg(serverSock->peerName())
+                                      .arg(serverSock->peerPort());
     return ret;
 }
 
@@ -172,13 +207,17 @@ void MythContext::LoadQtConfig(void)
     language = "";
     themecachedir = "";
 
-    m_height = QApplication::desktop()->height();
-    m_width = QApplication::desktop()->width();
+    m_xbase = GetNumSetting("GuiOffsetX", 0);
+    m_ybase = GetNumSetting("GuiOffsetY", 0);
 
-    if (GetNumSetting("GuiWidth") > 0)
-        m_width = GetNumSetting("GuiWidth");
-    if (GetNumSetting("GuiHeight") > 0)
-        m_height = GetNumSetting("GuiHeight");
+    m_width = GetNumSetting("GuiWidth", 0);
+    m_height = GetNumSetting("GuiHeight", 0);
+
+    if (m_width <= 0 || m_height <= 0)
+    {
+        m_height = QApplication::desktop()->height();
+        m_width = QApplication::desktop()->width();
+    }
 
     if (m_qtThemeSettings)
         delete m_qtThemeSettings;
@@ -194,13 +233,16 @@ void MythContext::LoadQtConfig(void)
     m_qtThemeSettings->ReadSettings(themedir);
     m_themeloaded = false;
 
+    if (m_backgroundimage)
+        delete m_backgroundimage;
+    m_backgroundimage = NULL;
+
     InitializeScreenSettings();
 }
 
 void MythContext::UpdateImageCache(void)
 {
     imageCache.clear();
-    pixmapCache.clear();
 
     ClearOldImageCache();
     CacheThemeImages();
@@ -355,6 +397,13 @@ void MythContext::CacheThemeImagesDirectory(const QString &dir)
     delete caching;
 }
 
+
+void MythContext::GetScreenSettings(float &wmult, float &hmult)
+{
+    wmult = m_wmult;
+    hmult = m_hmult;
+}
+
 void MythContext::GetScreenSettings(int &width, float &wmult, 
                                     int &height, float &hmult)
 {
@@ -365,16 +414,47 @@ void MythContext::GetScreenSettings(int &width, float &wmult,
     hmult = m_hmult;
 }
 
-void MythContext::InitializeScreenSettings(void)
+
+void MythContext::GetScreenSettings(int &xbase, int &width, float &wmult, 
+                                    int &ybase, int &height, float &hmult)
 {
+    xbase  = m_xbase;
+    ybase  = m_ybase;
+    
+    height = m_screenheight;
+    width = m_screenwidth;
+
+    wmult = m_wmult;
+    hmult = m_hmult;
+}
+
+void MythContext::InitializeScreenSettings()
+{
+    int x = 0, y = 0, w = 0, h = 0;
+
+#ifndef QWS
+    GetMythTVGeometry(qt_xdisplay(), qt_xscreen(), &x, &y, &w, &h);
+#endif
+
+    m_xbase = x + GetNumSetting("GuiOffsetX", 0);
+    m_ybase = y + GetNumSetting("GuiOffsetY", 0);
+
     int height = GetNumSetting("GuiHeight", m_height);
     int width = GetNumSetting("GuiWidth", m_width);
 
-    if (height == 0)
-        height = m_height;
-    if (width == 0)
-        width = m_width;
-
+    if (w != 0 && h != 0 && height == 0 && width == 0)
+    {
+        width = w;
+        height = h;
+    }
+    else
+    {
+        if (height == 0)
+            height = m_height;
+        if (width == 0)
+            width = m_width;
+    }
+    
     if (height < 160 || width < 160)
     {
         cerr << "Somehow, your screen size settings are bad.\n";
@@ -392,6 +472,10 @@ void MythContext::InitializeScreenSettings(void)
     m_hmult = height / 600.0;
     m_screenwidth = width;   
     m_screenheight = height;
+
+    bigfontsize = GetNumSetting("QtFontBig", 25);
+    mediumfontsize = GetNumSetting("QtFontMedium", 16);
+    smallfontsize = GetNumSetting("QtFontSmall", 12);
 }
 
 QString MythContext::FindThemeDir(QString themename)
@@ -490,28 +574,6 @@ QString MythContext::DBErrorMessage(const QSqlError& err)
         .arg(err.databaseText());
 }
 
-bool MythContext::CheckDBVersion(void)
-{
-    QString DBSchemaVer = GetSetting("DBSchemaVer", "0");
-
-    int dbversion = DBSchemaVer.toInt();
-
-    int appschema = atoi(MYTH_SCHEMA_VERSION);
-
-    if (dbversion < appschema)
-    {
-        cerr << "Your current database schema version is: " << dbversion
-             << "\nbut this application requires version "
-             << MYTH_SCHEMA_VERSION << " or higher.\n";
-        cerr << "You should update your database with the latest\n"
-             << "changes in the 'database' directory. See section 6\n"
-             << "of the MythTV documentation for more information.\n\n";
-
-        return false;
-    }
-    return true;
-}
-
 void MythContext::SaveSetting(QString key, int newValue)
 {
     QString strValue = QString("%1").arg(newValue);
@@ -687,8 +749,8 @@ void MythContext::ThemeWidget(QWidget *widget)
     if (m_themeloaded)
     {
         widget->setPalette(m_palette);
-        if (m_backgroundimage.width() > 0)
-            widget->setPaletteBackgroundPixmap(m_backgroundimage);
+        if (m_backgroundimage->width() > 0)
+            widget->setPaletteBackgroundPixmap(*m_backgroundimage);
         return;
     }
 
@@ -706,7 +768,7 @@ void MythContext::ThemeWidget(QWidget *widget)
         if (bgpixmap)
         {
             widget->setPaletteBackgroundPixmap(*bgpixmap);
-            m_backgroundimage = *bgpixmap;
+            m_backgroundimage = new QPixmap(*bgpixmap);
         }
     }
     else if (m_qtThemeSettings->GetSetting("TiledBackgroundPixmap") != "")
@@ -728,7 +790,7 @@ void MythContext::ThemeWidget(QWidget *widget)
             tmp.drawTiledPixmap(0, 0, width, height, *bgpixmap);
             tmp.end();
 
-            m_backgroundimage = background;
+            m_backgroundimage = new QPixmap(background);
             widget->setPaletteBackgroundPixmap(background);
         }
     }
@@ -822,19 +884,9 @@ QPixmap *MythContext::LoadScalePixmap(QString filename, bool fromcache)
         QFile cachecheck(cachefilepath);
         if (cachecheck.exists() && fromcache)
         {
-            //if (pixmapCache.contains(cachefilepath))
-            //{
-            //    return new QPixmap(pixmapCache[cachefilepath]);
-            //}
-
             QPixmap *ret = new QPixmap(cachefilepath);
             if (ret)
-            {
-                //pixmapCache[cachefilepath] = *ret;
                 return ret;
-            }
-
-            delete ret;
         }
     }
 
@@ -917,18 +969,21 @@ void MythContext::SetSetting(const QString &key, const QString &newValue)
     m_settings->SetSetting(key, newValue);
 }
 
-void MythContext::SendReceiveStringList(QStringList &strlist)
+bool MythContext::SendReceiveStringList(QStringList &strlist)
 {
     if (!serverSock)
         ConnectToMasterServer();
 
+    if (!serverSock)
+        return false;
+
     serverSockLock.lock();
     expectingReply = true;
 
     WriteStringList(serverSock, strlist);
-    ReadStringList(serverSock, strlist);
+    bool ok = ReadStringList(serverSock, strlist);
 
-    while (strlist[0] == "BACKEND_MESSAGE")
+    while (ok && strlist[0] == "BACKEND_MESSAGE")
     {
         // oops, not for us
         QString message = strlist[1];
@@ -937,11 +992,28 @@ void MythContext::SendReceiveStringList(QStringList &strlist)
         MythEvent me(message, extra);
         dispatch(me);
 
-        ReadStringList(serverSock, strlist);
+        ok = ReadStringList(serverSock, strlist);
+    }
+
+    if (!ok)
+    {
+        qApp->lock();
+        cout << "Connection to backend server lost\n";
+        MythPopupBox::showOkPopup(mainWindow, "connection failure",
+                                  tr("The connection to the master backend "
+                                     "server has gone away for some reason.. "
+                                     "Is it running?"));
+
+        serverSock->close();
+        delete serverSock;
+        serverSock = NULL;
+        qApp->unlock();
     }
 
     expectingReply = false;
     serverSockLock.unlock();
+
+    return ok;
 }
 
 void MythContext::readSocket(void)
@@ -951,10 +1023,12 @@ void MythContext::readSocket(void)
 
     serverSockLock.lock();
 
-    while (serverSock->bytesAvailable() > 0)
+    while (serverSock->state() == QSocket::Connected &&
+           serverSock->bytesAvailable() > 0)
     {
         QStringList strlist;
-        ReadStringList(serverSock, strlist);
+        if (!ReadStringList(serverSock, strlist))
+            continue;
 
         QString prefix = strlist[0];
         QString message = strlist[1];
@@ -1007,59 +1081,28 @@ void MythContext::dispatchNow(MythEvent &e)
     }
 }
 
-void MythContext::LCDconnectToHost(QString hostname, unsigned int port)
+QFont MythContext::GetBigFont(void)
 {
-    lcd_device->connectToHost(hostname, port);
+    return QFont("Arial", (int)(gContext->GetBigFontSize() * m_hmult),
+                 QFont::Bold);
 }
 
-void MythContext::LCDswitchToTime()
+QFont MythContext::GetMediumFont(void)
 {
-    lcd_device->switchToTime();
+    return QFont("Arial", (int)(gContext->GetMediumFontSize() * m_hmult),
+                 QFont::Bold);
 }
 
-void MythContext::LCDswitchToMusic(QString artist, QString track)
+QFont MythContext::GetSmallFont(void)
 {
-    lcd_device->switchToMusic(artist, track);
-}
-
-void MythContext::LCDsetLevels(int numb_levels, float *levels)
-{
-    lcd_device->setLevels(numb_levels, levels);
-}
-
-void MythContext::LCDswitchToChannel(QString channum, 
-                                     QString title, 
-                                     QString subtitle)
-{
-    lcd_device->switchToChannel(channum, title, subtitle);
-}
-
-void MythContext::LCDsetChannelProgress(float percentViewed)
-{
-    lcd_device->setChannelProgress(percentViewed);
-}
-
-void MythContext::LCDswitchToNothing()
-{
-    lcd_device->switchToNothing();
-}
-
-void MythContext::LCDpopMenu(QString menu_choice, QString menu_title)
-{
-    lcd_device->popMenu(menu_choice, menu_title);
-}
-
-void MythContext::LCDdestroy()
-{
-    lcd_device->shutdown();
-    delete lcd_device;
-    lcd_device = NULL;
+    return QFont("Arial", (int)(gContext->GetSmallFontSize() * m_hmult),
+                 QFont::Bold);
 }
 
 QString MythContext::GetLanguage(void)
 {
     if (language == QString::null || language == "")
-        language = GetSetting("Language").lower();
+        language = GetSetting("Language", "EN").lower();
 
     return language;
 }
