@@ -2,6 +2,7 @@
 #define MYTHCONTEXT_H_
 
 #include <qstring.h>
+#include <qdatetime.h>
 #include <qpixmap.h>
 #include <qpalette.h>
 #include <qobject.h>
@@ -20,8 +21,22 @@ class QSocket;
 class LCD;
 class MythMainWindow;
 
-#define VERBOSE(args...) \
-if (print_verbose_messages) \
+enum VerboseMask {
+    VB_IMPORTANT = 0x0001,
+    VB_GENERAL   = 0x0002,
+    VB_RECORD    = 0x0004,
+    VB_PLAYBACK  = 0x0008,
+    VB_CHANNEL   = 0x0010,
+    VB_OSD       = 0x0020,
+    VB_FILE      = 0x0040,
+    VB_SCHEDULE  = 0x0080,
+    VB_NETWORK   = 0x0100,
+    VB_NONE      = 0x0000,
+    VB_ALL       = 0xffff
+};
+
+#define VERBOSE(mask,args...) \
+if ((print_verbose_messages & mask) != 0) \
     cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") \
          << " " << args << endl;
 
@@ -52,23 +67,22 @@ class MythEvent : public QCustomEvent
     QString extradata;
 };
 
-#define MYTH_BINARY_VERSION "0.11.08132003-1"
-#define MYTH_SCHEMA_VERSION "1003"
+#define MYTH_BINARY_VERSION "0.12.10132003-1"
 
-extern bool print_verbose_messages;
+extern int print_verbose_messages;
 
 class MythContext : public QObject
 {
     Q_OBJECT
   public:
-    MythContext(const QString &binversion, bool gui = true);
+    MythContext(const QString &binversion, bool gui = true, bool lcd = true);
     virtual ~MythContext();
 
     QString GetMasterHostPrefix(void);
 
     QString GetHostName(void) { return m_localhostname; }
 
-    void ConnectToMasterServer(void);
+    bool ConnectToMasterServer(void);
     bool ConnectServer(const QString &hostname, int port);
 
     QString GetInstallPrefix() { return m_installprefix; }
@@ -80,7 +94,11 @@ class MythContext : public QObject
     void LoadQtConfig(void);
     void UpdateImageCache(void);
 
-    void GetScreenSettings(int &width, float &wmult, int &height, float &hmult);
+    void GetScreenSettings(float &wmult, float &hmult);
+    void GetScreenSettings(int &width, float &wmult,
+                           int &height, float &hmult);
+    void GetScreenSettings(int &xbase, int &width, float &wmult,
+                           int &ybase, int &height, float &hmult);
    
     QString FindThemeDir(QString themename);
     QString GetThemeDir(void) { return m_themepathname; }
@@ -90,8 +108,6 @@ class MythContext : public QObject
     static void DBError(QString where, const QSqlQuery& query);
     static QString DBErrorMessage(const QSqlError& err);
 
-    bool CheckDBVersion(void);
-
     Settings *settings() { return m_settings; }
     Settings *qtconfig() { return m_qtThemeSettings; }
 
@@ -107,9 +123,13 @@ class MythContext : public QObject
 
     void SetSetting(const QString &key, const QString &newValue);
 
-    int GetBigFontSize() { return GetNumSetting("QtFontBig", 25); }
-    int GetMediumFontSize() { return GetNumSetting("QtFontMedium", 16); }
-    int GetSmallFontSize() { return GetNumSetting("QtFontSmall", 12); }
+    int GetBigFontSize() { return bigfontsize; }
+    int GetMediumFontSize() { return mediumfontsize; }
+    int GetSmallFontSize() { return smallfontsize; }
+
+    QFont GetBigFont();
+    QFont GetMediumFont();
+    QFont GetSmallFont();
 
     QString GetLanguage(void);
 
@@ -123,26 +143,17 @@ class MythContext : public QObject
     void dispatch(MythEvent &e);
     void dispatchNow(MythEvent &e);
 
-    void SendReceiveStringList(QStringList &strlist);
+    bool SendReceiveStringList(QStringList &strlist);
 
     QImage *CacheRemotePixmap(const QString &url, bool needevents = true);
 
     void SetMainWindow(MythMainWindow *mainwin);
     MythMainWindow *GetMainWindow(void);
 
-    void LCDconnectToHost(QString hostname, unsigned int port);
-    void LCDswitchToTime();
-    void LCDswitchToMusic(QString artist, QString track);
-    void LCDsetLevels(int numb_levels, float *levels);
-    void LCDswitchToChannel(QString channum = "", QString title = "", 
-                            QString subtitle = "");
-    void LCDsetChannelProgress(float percentViewed);
-    void LCDswitchToNothing();
-    void LCDpopMenu(QString menu_choice, QString menu_title);
-    void LCDdestroy();
+    LCD *GetLCDDevice(void) { return lcd_device; }
 
     bool TestPopupVersion(const QString &name, const QString &libversion,
-                          const QString &pluginversion);	
+                          const QString &pluginversion);
   private slots:
     void readSocket();
 
@@ -162,9 +173,10 @@ class MythContext : public QObject
 
     bool m_themeloaded;
     QString m_themepathname;
-    QPixmap m_backgroundimage;
+    QPixmap *m_backgroundimage;
     QPalette m_palette;
 
+    int m_xbase, m_ybase;
     int m_height, m_width;
 
     QSocket *serverSock;
@@ -179,7 +191,6 @@ class MythContext : public QObject
     QMutex dbLock;
 
     QMap<QString, QImage> imageCache;
-    QMap<QString, QPixmap> pixmapCache;
 
     LCD *lcd_device;
 
@@ -191,6 +202,8 @@ class MythContext : public QObject
     int m_screenwidth, m_screenheight;
 
     QString themecachedir;
+
+    int bigfontsize, mediumfontsize, smallfontsize; 
 };
 
 extern MythContext *gContext;
