@@ -41,6 +41,23 @@ void MythComboBox::keyPressEvent(QKeyEvent *e)
     }
 }
 
+void MythComboBox::focusInEvent(QFocusEvent *e)
+{
+    emit changeHelpText(helptext);
+    emit gotFocus();
+
+    QColor highlight = colorGroup().highlight();
+
+    this->setPaletteBackgroundColor(highlight);
+    QComboBox::focusInEvent(e);
+}
+
+void MythComboBox::focusOutEvent(QFocusEvent *e)
+{
+    this->unsetPalette();
+    QComboBox::focusOutEvent(e);
+}
+
 void MythCheckBox::keyPressEvent(QKeyEvent* e)
 {
     switch (e->key()) {
@@ -61,18 +78,18 @@ void MythCheckBox::keyPressEvent(QKeyEvent* e)
 
 void MythCheckBox::focusInEvent(QFocusEvent *e)
 {
-     emit changeHelpText(helptext);
+    emit changeHelpText(helptext);
 
-     QColor highlight = colorGroup().highlight();
+    QColor highlight = colorGroup().highlight();
 
-     this->setPaletteBackgroundColor(highlight);
-     QCheckBox::focusInEvent(e);
+    this->setPaletteBackgroundColor(highlight);
+    QCheckBox::focusInEvent(e);
 }
 
 void MythCheckBox::focusOutEvent(QFocusEvent *e)
 {
-     this->unsetPalette();
-     QCheckBox::focusOutEvent(e);
+    this->unsetPalette();
+    QCheckBox::focusOutEvent(e);
 }
 
 bool MythSpinBox::eventFilter(QObject* o, QEvent* e)
@@ -80,7 +97,15 @@ bool MythSpinBox::eventFilter(QObject* o, QEvent* e)
     (void)o;
 
     if (e->type() == QEvent::FocusIn)
+    {
+        QColor highlight = colorGroup().highlight();
+        editor()->setPaletteBackgroundColor(highlight);
         emit(changeHelpText(helptext));
+    }
+    else if (e->type() == QEvent::FocusOut)
+    {
+        editor()->unsetPalette();
+    }
 
     if (e->type() != QEvent::KeyPress)
         return FALSE;
@@ -107,6 +132,22 @@ bool MythSpinBox::eventFilter(QObject* o, QEvent* e)
     return TRUE;
 }
 
+void MythSpinBox::focusInEvent(QFocusEvent *e)
+{
+    emit changeHelpText(helptext);
+
+    QColor highlight = colorGroup().highlight();
+
+    this->setPaletteBackgroundColor(highlight);
+    QSpinBox::focusInEvent(e);
+}
+
+void MythSpinBox::focusOutEvent(QFocusEvent *e)
+{
+    this->unsetPalette();
+    QSpinBox::focusOutEvent(e);
+}
+
 void MythSlider::keyPressEvent(QKeyEvent* e)
 {
     switch (e->key()) {
@@ -131,6 +172,22 @@ void MythSlider::keyPressEvent(QKeyEvent* e)
     }
 }
 
+void MythSlider::focusInEvent(QFocusEvent *e)
+{
+    emit changeHelpText(helptext);
+
+    QColor highlight = colorGroup().highlight();
+
+    this->setPaletteBackgroundColor(highlight);
+    QSlider::focusInEvent(e);
+}
+
+void MythSlider::focusOutEvent(QFocusEvent *e)
+{
+    this->unsetPalette();
+    QSlider::focusOutEvent(e);
+}
+
 void MythLineEdit::keyPressEvent(QKeyEvent *e)
 {
     switch (e->key()) {
@@ -149,7 +206,8 @@ void MythLineEdit::keyPressEvent(QKeyEvent *e)
     }
 }
 
-void MythLineEdit::setText(const QString& text) {
+void MythLineEdit::setText(const QString& text) 
+{
     // Don't mess with the cursor position; it causes
     // counter-intuitive behaviour due to interactions with the
     // communication with the settings stuff
@@ -159,6 +217,22 @@ void MythLineEdit::setText(const QString& text) {
     setCursorPosition(pos);
 }
 
+void MythLineEdit::focusInEvent(QFocusEvent *e)
+{
+    emit changeHelpText(helptext);
+
+    QColor highlight = colorGroup().highlight();
+
+    this->setPaletteBackgroundColor(highlight);
+    QLineEdit::focusInEvent(e);
+}
+
+void MythLineEdit::focusOutEvent(QFocusEvent *e)
+{
+    this->unsetPalette();
+    QLineEdit::focusOutEvent(e);
+}
+
 // thor feb 18 2003
 
 MythRemoteLineEdit::MythRemoteLineEdit(QFont *a_font, QWidget * parent, const char * name)
@@ -500,7 +574,7 @@ void MythRemoteLineEdit::keyPressEvent(QKeyEvent *e)
             //
 
         case Key_1:
-            cycleKeys("_X-/.?()1");
+            cycleKeys("_X%-/.?()1");
             handled = true;
             break;
 
@@ -650,17 +724,23 @@ void MythRemoteLineEdit::focusInEvent(QFocusEvent *e)
 {
     emit changeHelpText(helptext);
     emit gotFocus();    //perhaps we need to save a playlist?
+
+    QColor highlight = colorGroup().highlight();
+
+    this->setPaletteBackgroundColor(highlight);
+
     QTextEdit::focusInEvent(e);
 }
 
 
 void MythRemoteLineEdit::focusOutEvent(QFocusEvent *e)
 {
+    this->unsetPalette();
+
     emit lostFocus();
     QTextEdit::focusOutEvent(e);
 }
 
-
 MythRemoteLineEdit::~MythRemoteLineEdit()
 {
     if(cycle_timer)
@@ -669,9 +749,6 @@ MythRemoteLineEdit::~MythRemoteLineEdit()
     }
 }
 
-
-
-
 void MythTable::keyPressEvent(QKeyEvent *e)
 {
     bool handled = false;
@@ -1059,6 +1136,7 @@ void MythListView::keyPressEvent(QKeyEvent *e)
     }
 
     QListView::keyPressEvent(e);
+    emit unhandledKeyPress(e);
 }
 
 void MythListView::focusInEvent(QFocusEvent *e)
