@@ -27,6 +27,8 @@ ThemedMenu::ThemedMenu(const char *cdir, const char *menufile,
 {
     ignorekeys = false;
 
+    lcddev = gContext->GetLCDDevice();
+
     QString dir = QString(cdir) + "/";
     QString filename = dir + "theme.xml";
 
@@ -111,6 +113,7 @@ void ThemedMenu::parseBackground(QString dir, QDomElement &element)
     bool hasarea = false;
 
     spreadbuttons = true;
+    visiblerowlimit = 6;	// the old default
 
     QString type = element.attribute("style", "");
     if (type == "tiled")
@@ -142,6 +145,10 @@ void ThemedMenu::parseBackground(QString dir, QDomElement &element)
                 if (val == "no")
                     spreadbuttons = false;
             }
+            else if (info.tagName() == "visiblerowlimit")
+            {
+                visiblerowlimit = atoi(getFirstText(info).ascii());
+            }
             else
             {
                 cerr << "Unknown tag " << info.tagName() << " in background\n";
@@ -178,9 +185,9 @@ void ThemedMenu::parseBackground(QString dir, QDomElement &element)
     }
 }
 
-void ThemedMenu::parseShadow(QDomElement &element)
+void ThemedMenu::parseShadow(TextAttributes &attributes, QDomElement &element)
 {
-    hasshadow = true;
+    attributes.hasshadow = true;
 
     bool hascolor = false;
     bool hasoffset = false;
@@ -194,19 +201,21 @@ void ThemedMenu::parseShadow(QDomElement &element)
         {
             if (info.tagName() == "color")
             {
-                shadowColor = QColor(getFirstText(info));
+                attributes.shadowColor = QColor(getFirstText(info));
                 hascolor = true;
             }
             else if (info.tagName() == "offset")
             {
-                shadowOffset = parsePoint(getFirstText(info));
-                shadowOffset.setX((int)(shadowOffset.x() * wmult));
-                shadowOffset.setY((int)(shadowOffset.y() * hmult));
+                attributes.shadowOffset = parsePoint(getFirstText(info));
+                attributes.shadowOffset.setX((int)(attributes.shadowOffset.x() *
+                                             wmult));
+                attributes.shadowOffset.setY((int)(attributes.shadowOffset.y() *
+                                             hmult));
                 hasoffset = true;
             }
             else if (info.tagName() == "alpha")
             {
-                shadowalpha = atoi(getFirstText(info).ascii());
+                attributes.shadowalpha = atoi(getFirstText(info).ascii());
                 hasalpha = true;
             }
             else
@@ -236,9 +245,9 @@ void ThemedMenu::parseShadow(QDomElement &element)
     }
 }
 
-void ThemedMenu::parseOutline(QDomElement &element)
+void ThemedMenu::parseOutline(TextAttributes &attributes, QDomElement &element)
 {
-    hasoutline = true;
+    attributes.hasoutline = true;
 
     bool hascolor = false;
     bool hassize = false;
@@ -251,13 +260,13 @@ void ThemedMenu::parseOutline(QDomElement &element)
         {
             if (info.tagName() == "color")
             {
-                outlineColor = QColor(getFirstText(info));
+                attributes.outlineColor = QColor(getFirstText(info));
                 hascolor = true;
             }
             else if (info.tagName() == "size")
             {
-                outlinesize = atoi(getFirstText(info).ascii());
-                outlinesize = (int)(outlinesize * hmult);
+                attributes.outlinesize = atoi(getFirstText(info).ascii());
+                attributes.outlinesize = (int)(attributes.outlinesize * hmult);
                 hassize = true;
             }
             else
@@ -281,7 +290,7 @@ void ThemedMenu::parseOutline(QDomElement &element)
     }
 }
 
-void ThemedMenu::parseText(QDomElement &element)
+void ThemedMenu::parseText(TextAttributes &attributes, QDomElement &element)
 {
     bool hasarea = false;
 
@@ -299,15 +308,22 @@ void ThemedMenu::parseText(QDomElement &element)
             if (info.tagName() == "area") 
             {
                 hasarea = true;
-                textRect = parseRect(getFirstText(info));
-                textRect.moveTopLeft(QPoint((int)(textRect.x() * wmult),
-                                            (int)(textRect.y() * hmult)));
-                textRect.setWidth((int)(textRect.width() * wmult));
-                textRect.setHeight((int)(textRect.height() * hmult));
-                textRect = QRect(textRect.x(), textRect.y(),
-                                 buttonnormal->width() - textRect.width() - 
-                                 textRect.x(), buttonnormal->height() - 
-                                 textRect.height() - textRect.y());
+                attributes.textRect = parseRect(getFirstText(info));
+                attributes.textRect.moveTopLeft(QPoint(
+                                       (int)(attributes.textRect.x() * wmult),
+                                       (int)(attributes.textRect.y() * hmult)));
+                attributes.textRect.setWidth((int)
+                                       (attributes.textRect.width() * wmult));
+                attributes.textRect.setHeight((int)
+                                       (attributes.textRect.height() * hmult));
+                attributes.textRect = QRect(attributes.textRect.x(), 
+                                            attributes.textRect.y(),
+                                            buttonnormal->width() - 
+                                            attributes.textRect.width() - 
+                                            attributes.textRect.x(), 
+                                            buttonnormal->height() - 
+                                            attributes.textRect.height() - 
+                                            attributes.textRect.y());
             }
             else if (info.tagName() == "fontsize")
             {
@@ -329,20 +345,21 @@ void ThemedMenu::parseText(QDomElement &element)
             }
             else if (info.tagName() == "color")
             {
-                textColor = QColor(getFirstText(info));
+                attributes.textColor = QColor(getFirstText(info));
             }
             else if (info.tagName() == "centered")
             {
                 if (getFirstText(info) == "yes")
-                    textflags = Qt::AlignTop | Qt::AlignHCenter | WordBreak;
+                    attributes.textflags = Qt::AlignTop | Qt::AlignHCenter | 
+                                           WordBreak;
             } 
             else if (info.tagName() == "outline")
             {
-                parseOutline(info);
+                parseOutline(attributes, info);
             }
             else if (info.tagName() == "shadow")
             {
-                parseShadow(info);
+                parseShadow(attributes, info);
             }
             else
             {
@@ -352,8 +369,7 @@ void ThemedMenu::parseText(QDomElement &element)
         }
     }
 
-    font = QFont(fontname, (int)(fontsize * hmult), weight, italic);
-    setFont(font);
+    attributes.font = QFont(fontname, (int)(fontsize * hmult), weight, italic);
 
     if (!hasarea)
     {
@@ -366,11 +382,10 @@ void ThemedMenu::parseButtonDefinition(QString dir, QDomElement &element)
 {
     bool hasnormal = false;
     bool hasactive = false;
+    bool hasactivetext = false;
 
     QString setting;
 
-    textflags = Qt::AlignTop | Qt::AlignLeft | WordBreak;
-
     for (QDomNode child = element.firstChild(); !child.isNull();
          child = child.nextSibling())
     {
@@ -393,11 +408,22 @@ void ThemedMenu::parseButtonDefinition(QString dir, QDomElement &element)
             {
                 if (!hasnormal)
                 {
-                    cerr << "The 'normal' tag needs to come before the 'text' "
-                         << "tag\n";
+                    cerr << "The 'normal' tag needs to come before the "
+                         << "'normaltext' tag\n";
                     exit(0);
                 }
-                parseText(info);
+                parseText(normalAttributes, info);
+            }
+            else if (info.tagName() == "activetext")
+            {
+                if (!hasactive)
+                {
+                    cerr << "The 'active' tag needs to come before the "
+                         << "'activetext' tag\n";
+                    exit(0);
+                }
+                parseText(activeAttributes, info);
+                hasactivetext = true;
             }
             else if (info.tagName() == "watermarkposition")
             {
@@ -417,12 +443,18 @@ void ThemedMenu::parseButtonDefinition(QString dir, QDomElement &element)
         cerr << "No normal button image defined\n";
         exit(0);
     }
+
     if (!hasactive)
     {
         cerr << "No active button image defined\n";
         exit(0);
     }
 
+    if (!hasactivetext)
+    {
+        activeAttributes = normalAttributes;
+    }
+
     watermarkPos.setX((int)(watermarkPos.x() * wmult));
     watermarkPos.setY((int)(watermarkPos.y() * hmult));
 
@@ -697,10 +729,17 @@ void ThemedMenu::setDefaults(void)
 {
     logo = NULL;
     buttonnormal = buttonactive = NULL;
-    textflags = Qt::AlignTop | Qt::AlignLeft | WordBreak;
-    textColor = QColor(white);
-    hasoutline = false;
-    hasshadow = false;
+
+    normalAttributes.textflags = Qt::AlignTop | Qt::AlignLeft | WordBreak;
+    normalAttributes.textColor = QColor(white);
+    normalAttributes.hasoutline = false;
+    normalAttributes.hasshadow = false;
+
+    activeAttributes.textflags = Qt::AlignTop | Qt::AlignLeft | WordBreak;
+    activeAttributes.textColor = QColor(white);
+    activeAttributes.hasoutline = false;
+    activeAttributes.hasshadow = false;
+
     titleIcons.clear();
     titleText = "";
     curTitle = NULL;
@@ -858,6 +897,10 @@ void ThemedMenu::parseThemeButton(QDomElement &element)
             {
                 addit = findDepends(getFirstText(info));
             }
+            else if (info.tagName() == "dependssetting")
+            {
+                addit = gContext->GetNumSetting(getFirstText(info));
+            }
             else
             {
                 cerr << "Unknown tag " << info.tagName() << " in button\n";
@@ -988,9 +1031,29 @@ void ThemedMenu::parseMenu(QString menuname, int row, int col)
 
     drawInactiveButtons();
 
-    titleText = "MYTH-";
-    titleText +=  menumode;
-    gContext->LCDpopMenu(activebutton->text, titleText);
+    if (lcddev)
+    {
+        titleText = "MYTH-";
+        titleText += menumode;
+        QPtrList<LCDMenuItem> menuItems;
+        menuItems.setAutoDelete(true);
+        bool selected;
+
+        for (int r = 0; r < (int)buttonRows.size(); r++)
+        {
+            if (r == currentrow)
+                selected = true;
+            else
+                selected = false;
+
+            if (currentcolumn < buttonRows[r].numitems)
+                menuItems.append(new LCDMenuItem(selected, NOTCHECKABLE,
+                                 buttonRows[r].buttons[currentcolumn]->text));
+        }
+
+        if (!menuItems.isEmpty())
+            lcddev->switchToMenu(&menuItems, titleText);
+    }
 
     selection = "";
     update(menuRect());
@@ -1065,9 +1128,9 @@ void ThemedMenu::layoutButtons(void)
     }    
 
     // limit it to 6 items displayed at one time
-    if (columns * maxrows > 6)
+    if (columns * maxrows > visiblerowlimit)
     {
-        maxrows = 6 / columns;
+        maxrows = visiblerowlimit / columns;
     }
                              
     vector<ThemedButton>::iterator iter = buttonList.begin();
@@ -1338,6 +1401,7 @@ void ThemedMenu::paintWatermark(QPainter *p)
 void ThemedMenu::paintButton(unsigned int button, QPainter *p, bool erased,
                              bool drawinactive)
 {
+    TextAttributes attributes;
     ThemedButton *tbutton = &(buttonList[button]);
 
     if (!tbutton->visible)
@@ -1366,6 +1430,7 @@ void ThemedMenu::paintButton(unsigned int button, QPainter *p, bool erased,
     {
         buttonback = buttonactive;
         tbutton->status = 1;
+        attributes = activeAttributes;
     }
     else
     {
@@ -1377,6 +1442,7 @@ void ThemedMenu::paintButton(unsigned int button, QPainter *p, bool erased,
             return;
         }
         buttonback = buttonnormal;
+        attributes = normalAttributes;
     }
 
     QPixmap pix(cr.size());
@@ -1388,35 +1454,36 @@ void ThemedMenu::paintButton(unsigned int button, QPainter *p, bool erased,
 
     tmp.drawImage(newRect.topLeft(), *buttonback);
 
-    QRect buttonTextRect = textRect;
+    QRect buttonTextRect = attributes.textRect;
     buttonTextRect.moveBy(newRect.x(), newRect.y());
 
     QString message = tbutton->text;
-    QRect testBound = tmp.boundingRect(buttonTextRect, textflags, message);
+    QRect testBound = tmp.boundingRect(buttonTextRect, attributes.textflags, 
+                                       message);
 
     if (testBound.height() > buttonTextRect.height() && tbutton->altText != "")
         message = buttonList[button].altText;
 
-    if (hasshadow && shadowalpha > 0)
+    if (attributes.hasshadow && attributes.shadowalpha > 0)
     {
         QPixmap textpix(buttonTextRect.size());
         textpix.fill(QColor(color0));
         textpix.setMask(textpix.createHeuristicMask());
 
         QRect myrect = buttonTextRect;
-        myrect.moveTopLeft(shadowOffset);
+        myrect.moveTopLeft(attributes.shadowOffset);
 
         QPainter texttmp;
         texttmp.begin(&textpix, this);
-        texttmp.setPen(QPen(shadowColor, 1));
-        texttmp.setFont(font);
-        texttmp.drawText(myrect, textflags, message);
+        texttmp.setPen(QPen(attributes.shadowColor, 1));
+        texttmp.setFont(attributes.font);
+        texttmp.drawText(myrect, attributes.textflags, message);
         texttmp.end();
 
         texttmp.begin(textpix.mask());
         texttmp.setPen(QPen(Qt::color1, 1));
-        texttmp.setFont(font);
-        texttmp.drawText(myrect, textflags, message);
+        texttmp.setFont(attributes.font);
+        texttmp.drawText(myrect, attributes.textflags, message);
         texttmp.end();
 
         QImage im = textpix.convertToImage();
@@ -1428,7 +1495,8 @@ void ThemedMenu::paintButton(unsigned int button, QPainter *p, bool erased,
                uint *p = (uint *)im.scanLine(y) + x;
                QRgb col = *p;
                if (qAlpha(col) > 128)
-                   *p = qRgba(qRed(col), qGreen(col), qBlue(col), shadowalpha);
+                   *p = qRgba(qRed(col), qGreen(col), qBlue(col), 
+                              attributes.shadowalpha);
             }
         } 
 
@@ -1438,8 +1506,8 @@ void ThemedMenu::paintButton(unsigned int button, QPainter *p, bool erased,
         bitBlt(&pix, buttonTextRect.topLeft(), &textpix, myrect, Qt::CopyROP);
     }
 
-    tmp.setFont(font);
-    drawText(&tmp, buttonTextRect, textflags, message);
+    tmp.setFont(attributes.font);
+    drawText(&tmp, buttonTextRect, attributes, message);
 
     if (buttonList[button].buttonicon)
     {
@@ -1462,47 +1530,53 @@ void ThemedMenu::paintButton(unsigned int button, QPainter *p, bool erased,
     p->drawPixmap(cr.topLeft(), pix);
 }
 
-void ThemedMenu::drawText(QPainter *p, QRect &rect, int textflags, QString text)
+void ThemedMenu::drawText(QPainter *p, QRect &rect, TextAttributes attributes, 
+                          QString text)
 {
-    if (hasoutline)
+    if (attributes.hasoutline)
     {
         QRect outlinerect = rect;
 
-        p->setPen(QPen(outlineColor, 1));
-        outlinerect.moveBy(0 - outlinesize, 0 - outlinesize);
-        p->drawText(outlinerect, textflags, text);
+        p->setPen(QPen(attributes.outlineColor, 1));
+        outlinerect.moveBy(0 - attributes.outlinesize, 
+                           0 - attributes.outlinesize);
+        p->drawText(outlinerect, attributes.textflags, text);
 
-        for (int i = (0 - outlinesize + 1); i <= outlinesize; i++)
+        for (int i = (0 - attributes.outlinesize + 1); 
+             i <= attributes.outlinesize; i++)
         {
             outlinerect.moveBy(1, 0);
-            p->drawText(outlinerect, textflags, text);
+            p->drawText(outlinerect, attributes.textflags, text);
         }
 
-        for (int i = (0 - outlinesize + 1); i <= outlinesize; i++)
+        for (int i = (0 - attributes.outlinesize + 1); 
+             i <= attributes.outlinesize; i++)
         {
             outlinerect.moveBy(0, 1);
-            p->drawText(outlinerect, textflags, text);
+            p->drawText(outlinerect, attributes.textflags, text);
         }
   
-        for (int i = (0 - outlinesize + 1); i <= outlinesize; i++)
+        for (int i = (0 - attributes.outlinesize + 1); 
+             i <= attributes.outlinesize; i++)
         {
             outlinerect.moveBy(-1, 0);
-            p->drawText(outlinerect, textflags, text);
+            p->drawText(outlinerect, attributes.textflags, text);
         }
 
-        for (int i = (0 - outlinesize + 1); i <= outlinesize; i++)
+        for (int i = (0 - attributes.outlinesize + 1); 
+             i <= attributes.outlinesize; i++)
         {
             outlinerect.moveBy(0, -1);
-            p->drawText(outlinerect, textflags, text);
+            p->drawText(outlinerect, attributes.textflags, text);
         }
 
-        p->setPen(QPen(textColor, 1));
-        p->drawText(rect, textflags, text);
+        p->setPen(QPen(attributes.textColor, 1));
+        p->drawText(rect, attributes.textflags, text);
     }
     else
     {
-        p->setPen(QPen(textColor, 1));
-        p->drawText(rect, textflags, text);
+        p->setPen(QPen(attributes.textColor, 1));
+        p->drawText(rect, attributes.textflags, text);
     }
 }
 
@@ -1548,13 +1622,6 @@ void ThemedMenu::ReloadTheme(void)
     downarrow = NULL;
 
     gContext->GetScreenSettings(screenwidth, wmult, screenheight, hmult);
-
-    int x = 0, y = 0, w, h;
-#ifndef QWS
-    GetMythTVGeometry(qt_xdisplay(), qt_xscreen(), &x, &y, &w, &h);
-#endif
-
-    setGeometry(x, y, screenwidth, screenheight);
     setFixedSize(QSize(screenwidth, screenheight));
 
     setFont(QFont("Arial", (int)(gContext->GetMediumFontSize() * hmult),
@@ -1632,6 +1699,7 @@ void ThemedMenu::keyPressEvent(QKeyEvent *e)
         case Key_Space:
         {
             handleAction(activebutton->action);
+            lastbutton = NULL;
             handled = true;
             break;
         }
@@ -1643,6 +1711,7 @@ void ThemedMenu::keyPressEvent(QKeyEvent *e)
             else if (killable || e->state() == exitModifier)
                 done(0);
             handled = true;
+            lastbutton = NULL;
             break;
         }
         default: break;
@@ -1651,12 +1720,36 @@ void ThemedMenu::keyPressEvent(QKeyEvent *e)
     if (handled)
     {
         if (!buttonRows[currentrow].visible)
+        {
             makeRowVisible(currentrow, oldrow);
+            lastbutton = NULL;
+        }
 
         activebutton = buttonRows[currentrow].buttons[currentcolumn];
-        gContext->LCDpopMenu(activebutton->text, titleText);
+        if (lcddev)
+        {
+            // Build a list of the menu items
+            QPtrList<LCDMenuItem> menuItems;
+            menuItems.setAutoDelete(true);
+            bool selected;
+            for (int r = 0; r < (int)buttonRows.size(); r++)
+            {
+                if (r == currentrow)
+                    selected = true;
+                else
+                    selected = false;
+
+                if (currentcolumn < buttonRows[r].numitems)
+                    menuItems.append(new LCDMenuItem(selected, NOTCHECKABLE,
+                                   buttonRows[r].buttons[currentcolumn]->text));
+            }
+
+            if (!menuItems.isEmpty())
+                lcddev->switchToMenu(&menuItems, titleText);
+        }
         update(watermarkRect);
-        update(lastbutton->posRect);
+        if (lastbutton)
+            update(lastbutton->posRect);
         update(activebutton->posRect);
     }
     else
@@ -1734,6 +1827,13 @@ void ThemedMenu::handleAction(QString &action)
         menufiles.back().row = currentrow;
         menufiles.back().col = currentcolumn;
 
+        if (rest == "main_settings.xml" && 
+            gContext->GetNumSetting("SetupPinCodeRequired", 0) &&
+            !checkPinCode("SetupPinCodeTime", "SetupPinCode", "Setup Pin:"))
+        {
+            return;
+        }
+
         parseMenu(rest);
     }
     else if (action.left(6) == "UPMENU")
@@ -1758,6 +1858,11 @@ void ThemedMenu::handleAction(QString &action)
         QString rest = action.right(action.length() - 7);
         MythPluginManager::run_plugin(rest.stripWhiteSpace());
     }
+    else if (action.left(8) == "SHUTDOWN")
+    {
+        if (menulevel == 1)
+            done(0);
+    }
     else
     {
         selection = action;
@@ -1783,3 +1888,55 @@ bool ThemedMenu::findDepends(QString file)
 
     return false;
 }
+
+bool ThemedMenu::checkPinCode(QString timestamp_setting, 
+                              QString password_setting, QString text)
+{
+    QDateTime curr_time = QDateTime::currentDateTime();
+    QString last_time_stamp = gContext->GetSetting(timestamp_setting);
+    QString password = gContext->GetSetting(password_setting);
+
+    if (password.length() < 1)
+        return true;
+
+    if (last_time_stamp.length() < 1)
+    {
+        cerr << "themedmenu.o: Could not read password/pin time stamp. "
+             << "This is only an issue if it happens repeatedly. " << endl;
+    }
+    else
+    {
+        QDateTime last_time = QDateTime::fromString(last_time_stamp, 
+                                                    Qt::TextDate);
+        if (last_time.secsTo(curr_time) < 120)
+        {
+            last_time_stamp = curr_time.toString(Qt::TextDate);
+            gContext->SetSetting(timestamp_setting, last_time_stamp);
+            gContext->SaveSetting(timestamp_setting, last_time_stamp);
+            return true;
+        }
+    }
+
+    if (password.length() > 0)
+    {
+        bool ok = false;
+        MythPasswordDialog *pwd = new MythPasswordDialog(text, &ok, password,
+                                                     gContext->GetMainWindow());
+        pwd->exec();
+        delete pwd;
+        if (ok)
+        {
+            last_time_stamp = curr_time.toString(Qt::TextDate);
+            gContext->SetSetting(timestamp_setting, last_time_stamp);
+            gContext->SaveSetting(timestamp_setting, last_time_stamp);
+            return true;
+        }
+    }
+    else
+    {
+        return true;
+    }
+
+    return false;
+}
+
