@@ -527,6 +527,7 @@ class GenericTree
     GenericTree*  getChildByName(QString a_name);
     void          sortByString();
     void          sortBySelectable();
+    void          deleteAllChildren();
 
   private:
 
@@ -570,7 +571,7 @@ class UIManagedTreeListType : public UIType
     void    Draw(QPainter *, int drawlayer, int context);
     void    assignTreeData(GenericTree *a_tree);
     void    moveToNode(QValueList<int> route_of_branches);
-    void    moveToNodesFirstChild(QValueList<int> route_of_ranchs);
+    void    moveToNodesFirstChild(QValueList<int> route_of_branchs);
     QValueList <int> * getRouteToActive();
     bool    tryToSetActive(QValueList <int> route);
     void    setHighlightImage(QPixmap an_image){highlight_image = an_image;}
@@ -598,13 +599,16 @@ class UIManagedTreeListType : public UIType
     void    sortTreeByString(){if(my_tree_data) my_tree_data->sortByString(); }
     void    sortTreeBySelectable(){if(my_tree_data) my_tree_data->sortBySelectable();}
     GenericTree *getCurrentNode() { return current_node; }
+    void    setCurrentNode(GenericTree *a_node);
 
   public slots:
 
     bool    popUp();
     bool    pushDown();
-    bool    moveUp();
-    bool    moveDown();
+    bool    moveUp(bool do_refresh = true);
+    bool    moveDown(bool do_refresh = true);
+    bool    pageUp();
+    bool    pageDown();
     bool    nextActive(bool wrap_around, bool traverse_up_down);
     bool    prevActive(bool wrap_around, bool traverse_up_down);
     void    select();
@@ -619,6 +623,7 @@ class UIManagedTreeListType : public UIType
 
   private:
 
+    int   calculateEntriesInBin(int bin_number);
     bool  complexInternalNextPrevActive(bool forward_or_reverse, bool wrap_around);    
     QRect area;
     int   bins;
