@@ -18,6 +18,7 @@ extern "C" {
 #include <qimage.h>
 #include <qpainter.h>
 #include <qpixmap.h>
+#include <qfont.h>
 
 #ifdef USE_LIRC
 #include "lircevent.h"
@@ -39,19 +40,40 @@ bool WriteStringList(QSocket *socket, QStringList &list)
     payload.truncate(8);
     payload += utf8;
     size = payload.length();
-    // cerr << payload << endl; //DEBUG
+
+    if ((print_verbose_messages & VB_NETWORK) != 0)
+    {
+        QString msg = payload;
+
+        if (msg.length() > 58)
+        {
+            msg.truncate(55);
+            msg += "...";
+        }
+        VERBOSE(VB_NETWORK, msg);
+    }
+
+    unsigned int errorcount = 0;
+    bool retval = true;
+
     while (size > 0)
     {
         qApp->lock();
         int temp = socket->writeBlock(payload.data() + written, size);
         qApp->unlock();
-	// cerr << "  written: " << temp << endl; //DEBUG
+        // cerr << "  written: " << temp << endl; //DEBUG
         written += temp;
         size -= temp;
         if (size > 0)
         {
             printf("Partial WriteStringList %u\n", written);
             qApp->processEvents();
+
+            if (++errorcount > 50)
+            {
+                retval = false;
+                break;
+            }  
         }
     }
 
@@ -60,7 +82,7 @@ bool WriteStringList(QSocket *socket, QStringList &list)
         socket->flush();
     qApp->unlock();
 
-    return true;
+    return retval;
 }
 
 bool ReadStringList(QSocket *socket, QStringList &list)
@@ -70,6 +92,13 @@ bool ReadStringList(QSocket *socket, QStringList &list)
     qApp->lock();
     while (socket->waitForMore(5) < 8)
     {
+        if (socket->state() != QSocket::Connected)
+        {
+            // dunno if socket->state() wants the app lock.  be safe for now.
+            qApp->unlock();
+            return false;
+        }
+
         qApp->unlock();
         usleep(50);
         qApp->lock();
@@ -93,17 +122,17 @@ bool ReadStringList(QSocket *socket, QStringList &list)
         qApp->lock();
         int temp = socket->readBlock(utf8.data() + read, size);
         qApp->unlock();
-	// cerr << "  read: " << temp << endl; //DEBUG
+        // cerr << "  read: " << temp << endl; //DEBUG
         read += temp;
         size -= temp;
         if (size > 0)
-	{
-	    if (++zerocnt >= 100)
-	    {
-		printf("EOF readStringList %u\n", read);
-		break; 
-	    }
-	    usleep(50);
+        {
+            if (++zerocnt >= 100)
+            {
+                printf("EOF readStringList %u\n", read);
+                break; 
+            }
+            usleep(50);
             qApp->processEvents();
 
             if (zerocnt == 5)
@@ -120,11 +149,13 @@ bool ReadStringList(QSocket *socket, QStringList &list)
     return true;
 }
 
-void WriteBlock(QSocket *socket, void *data, int len)
+bool WriteBlock(QSocket *socket, void *data, int len)
 {
     int size = len;
     int written = 0;
 
+    unsigned int errorcount = 0;
+
     while (size > 0)
     {
         qApp->lock();
@@ -136,6 +167,9 @@ void WriteBlock(QSocket *socket, void *data, int len)
         {
             printf("Partial WriteBlock %u\n", written);
             qApp->processEvents();
+
+            if (++errorcount > 50)
+                return false;
         }
     }
 
@@ -146,6 +180,8 @@ void WriteBlock(QSocket *socket, void *data, int len)
 
     while (socket->bytesToWrite() >= (unsigned) written)
         usleep(50000);
+
+    return true;
 }
 
 int ReadBlock(QSocket *socket, void *data, int maxlen)
@@ -162,13 +198,13 @@ int ReadBlock(QSocket *socket, void *data, int maxlen)
         read += temp;
         size -= temp;
         if (size > 0)
-	{
-	    if (++zerocnt >= 100)
-	    {
-		printf("EOF ReadBlock %u\n", read);
-		break; 
-	    }
-	    usleep(50);
+        {
+            if (++zerocnt >= 100)
+            {
+                printf("EOF ReadBlock %u\n", read);
+                break; 
+            }
+            usleep(50);
             qApp->processEvents();
         }
     }
@@ -200,9 +236,9 @@ void GetMythTVGeometry(Display *dpy, int screen_num, int *x, int *y,
 {
     int event_base, error_base;
 
-    if( XineramaQueryExtension(dpy, &event_base, &error_base) &&
-        XineramaIsActive(dpy) ) {
-
+    if (XineramaQueryExtension(dpy, &event_base, &error_base) &&
+        XineramaIsActive(dpy)) 
+    {
         XineramaScreenInfo *xinerama_screens;
         XineramaScreenInfo *screen;
         int nr_xinerama_screens;
@@ -213,12 +249,15 @@ void GetMythTVGeometry(Display *dpy, int screen_num, int *x, int *y,
 
         printf("Found %d Xinerama Screens.\n", nr_xinerama_screens);
 
-        if( screen_nr > 0 && screen_nr < nr_xinerama_screens ) {
+        if(screen_nr > 0 && screen_nr < nr_xinerama_screens)
+        {
             screen = &xinerama_screens[screen_nr];
             printf("Using screen %d, %dx%d+%d+%d\n",
                    screen_nr, screen->width, screen->height, screen->x_org, 
                    screen->y_org );
-        } else {
+        } 
+        else 
+        {
             screen = &xinerama_screens[0];
             printf("Using first Xinerama screen, %dx%d+%d+%d\n",
                    screen->width, screen->height, screen->x_org, screen->y_org);
@@ -230,7 +269,9 @@ void GetMythTVGeometry(Display *dpy, int screen_num, int *x, int *y,
         *y = screen->y_org;
 
         XFree(xinerama_screens);
-    } else {
+    } 
+    else 
+    {
         *w = DisplayWidth(dpy, screen_num);
         *h = DisplayHeight(dpy, screen_num);
         *x = 0; *y = 0;
@@ -263,8 +304,36 @@ int myth_system(const QString &command, int flags)
 {
 #ifdef USE_LIRC
     LircEventLock lirc_lock(!(flags & MYTH_SYSTEM_DONT_BLOCK_LIRC));
+#else
+    (void)flags;
 #endif
 
     return system(command);
 }
 
+QString cutDownString(QString text, QFont *testFont, int maxwidth)
+{
+    QFontMetrics fm(*testFont);
+
+    int curFontWidth = fm.width(text);
+    if (curFontWidth > maxwidth)
+    {
+        QString testInfo = "";
+        curFontWidth = fm.width(testInfo);
+        int tmaxwidth = maxwidth - fm.width("LLL");
+        int count = 0;
+
+        while (curFontWidth < tmaxwidth)
+        {
+            testInfo = text.left(count);
+            curFontWidth = fm.width(testInfo);
+            count = count + 1;
+        }
+
+        testInfo = testInfo + "...";
+        text = testInfo;
+    }
+
+    return text;
+}
+
