@@ -282,6 +282,12 @@ bool NuppelVideoRecorder::GetPause(void)
     return (audiopaused && mainpaused && actuallypaused);
 }
 
+void NuppelVideoRecorder::WaitForPause(void)
+{
+    while (!GetPause())
+        usleep(5000);
+}
+
 void NuppelVideoRecorder::SetVideoFilters(QString &filters)
 {
     videoFilterList = filters;
@@ -319,7 +325,7 @@ bool NuppelVideoRecorder::SetupAVCodec(void)
 
     if (!mpa_codec)
     {
-        cout << "error finding codec\n";
+        cerr << "error finding codec: " << codec.ascii() << endl;
         return false;
     }
 
@@ -424,12 +430,11 @@ void NuppelVideoRecorder::Initialize(void)
         codec = "mjpeg";
         hardware_encode = true;
 
-        switch (hmjpg_hdecimation)
-        {
-            case 2: w = 352; break;
-            case 4: w = 176; break;
-            default: w = hmjpg_maxw; break;
-        }
+        if (MJPEGInit() != 0)
+            cerr << "Could not detect max width for hardware MJPEG card, "
+                    "falling back to default: " << hmjpg_maxw << endl;
+ 
+        w = hmjpg_maxw / hmjpg_hdecimation;
 
         if (ntsc)
         {
@@ -491,6 +496,7 @@ int NuppelVideoRecorder::AudioInit(bool skipdevice)
 {
     int afmt, afd;
     int frag, blocksize = 4096;
+    int tmp;
 
     if (!skipdevice)
     {
@@ -550,9 +556,12 @@ int NuppelVideoRecorder::AudioInit(bool skipdevice)
         lame_set_compression_ratio(gf, 11);
         lame_set_mode(gf, audio_channels == 2 ? STEREO : MONO);
         lame_set_num_channels(gf, audio_channels);
-        lame_set_out_samplerate(gf, audio_samplerate);
         lame_set_in_samplerate(gf, audio_samplerate);
-        lame_init_params(gf);
+        if((tmp = lame_init_params(gf)) != 0)
+        {
+            cerr << "lame_init_params error " << tmp << "\n";
+            compressaudio = false; 
+        }
 
         if (audio_bits != 16) 
         {
@@ -566,6 +575,48 @@ int NuppelVideoRecorder::AudioInit(bool skipdevice)
     return 0; 
 }
 
+int NuppelVideoRecorder::MJPEGInit(void)
+{
+    fd = open(videodevice.ascii(), O_RDWR);
+    if (fd <= 0)
+    {
+        cerr << "Can't open video device: " << videodevice << endl;
+        perror("open video:");
+        return 1;
+    }
+
+    struct video_capability vc;
+
+    memset(&vc, 0, sizeof(vc));
+
+    if (ioctl(fd, VIDIOCGCAP, &vc) < 0)
+    {
+        perror("VIDIOCGCAP:");
+        close(fd);
+        return 1;
+    }
+
+    close(fd);
+
+    if (vc.type & VID_TYPE_MJPEG_ENCODER)
+    {
+        if (vc.maxwidth >= 768)
+            hmjpg_maxw = 768;
+        else if (vc.maxwidth >= 704)
+            hmjpg_maxw = 704;
+        else
+            hmjpg_maxw = 640;
+    }
+    else
+    {
+        cerr << "Video device " << videodevice << " does not appear to have "
+                "hardware MJPEG capture capabilities." << endl;
+        return 1;
+    }
+
+    return 0; 
+}
+
 void NuppelVideoRecorder::InitFilters(void)
 {
     if (videoFilters.size() > 0)
@@ -577,8 +628,11 @@ void NuppelVideoRecorder::InitFilters(void)
     QStringList filters = QStringList::split(",", videoFilterList);
     for (QStringList::Iterator i = filters.begin(); i != filters.end(); i++)
     {
-        VideoFilter *filter = load_videoFilter((char *)((*i).ascii()), 
-                                               NULL);
+        QString filtname = (*i).section('=', 0, 0);
+        QString filtopts = (*i).section('=', 1, 0xffffffff,
+                                        QString::SectionIncludeTrailingSep);
+        VideoFilter *filter = load_videoFilter((char *)(filtname.ascii()), 
+                                               (char *)(filtopts.ascii()));
         if (filter != NULL)
             videoFilters.push_back(filter);
     }
@@ -765,13 +819,6 @@ void NuppelVideoRecorder::StartRecording(void)
 
     if ((vc.type & VID_TYPE_MJPEG_ENCODER) && hardware_encode)
     {
-        if (vc.maxwidth >= 768)
-            hmjpg_maxw = 768;
-        else if (vc.maxwidth >= 704)
-            hmjpg_maxw = 704;
-        else
-            hmjpg_maxw = 640;
-
         DoMJPEG();
         return;
     }
@@ -2634,7 +2681,9 @@ void NuppelVideoRecorder::WriteAudio(unsigned char *buf, int fnum, int timecode)
         if (fnum != (last_block+1)) 
         {
             audio_behind = fnum - (last_block+1);
-            printf("audio behind %d %d\n", last_block, fnum);
+            QString msg = QString("audio behind %1 %2")
+                          .arg(last_block).arg(fnum);
+            VERBOSE(VB_RECORD, msg);
         }
     }
 
@@ -2728,7 +2777,7 @@ void NuppelVideoRecorder::WriteAudio(unsigned char *buf, int fnum, int timecode)
     // 'uncountable' video frame drop -> material==worthless
     if (audio_behind > 0) 
     {
-        cerr << "audio behind\n";
+        VERBOSE(VB_RECORD, "audio behind");
         frameheader.frametype = 'A'; // audio frame
         frameheader.comptype  = 'N'; // output a nullframe with
         frameheader.packetlength = 0;
