@@ -54,7 +54,8 @@ class NuppelVideoRecorder : public RecorderBase
     void Pause(bool clear = true);
     void Unpause(void);
     bool GetPause(void); 
-    
+    void WaitForPause(void);   
+ 
     bool IsRecording(void);
    
     long long GetFramesWritten(void); 
@@ -94,6 +95,8 @@ class NuppelVideoRecorder : public RecorderBase
  private:
     void InitBuffers(void);
     void InitFilters(void);   
+
+    int MJPEGInit(void);
  
     int SpawnChildren(void);
     void KillChildren(void);
