@@ -352,7 +352,7 @@ void RingBuffer::Init(void)
     fill_min = -1;
 
     readblocksize = 128000;
-    requestedblocks = 0;
+    requestedbytes = 0;
 
     recorder_num = 0;
     remoteencoder = NULL;
@@ -364,6 +364,7 @@ void RingBuffer::Init(void)
     totalreadpos = readpos = 0;
 
     stopreads = false;
+    wanttoread = 0;
 
     pthread_rwlock_init(&rwlock, NULL);
 }
@@ -392,6 +393,7 @@ void RingBuffer::Start(void)
 {
     if (remotefile)
         remotefile->Start();
+
     if ((normalfile && writemode) || (!normalfile && !recorder_num))
     {
     }
@@ -413,13 +415,8 @@ void RingBuffer::Reset(void)
         }
         else
         {
-            delete tfw;
-            close(fd2);
-
-            tfw = new ThreadedFileWriter(filename.ascii(),
-                                         O_WRONLY|O_CREAT|O_TRUNC|O_LARGEFILE, 
-                                         0644);
-            fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE);
+            tfw->Seek(0, SEEK_SET);
+            lseek(fd2, 0, SEEK_SET);
         }
 
         totalwritepos = writepos = 0;
@@ -462,7 +459,8 @@ int RingBuffer::safe_read(int fd, void *data, unsigned sz)
                 break;
 
             zerocnt++;
-            if (zerocnt >= 50) // 3 second timeout with usleep(60000)
+            if ((normalfile && zerocnt > 15) || 
+                zerocnt >= 50) // 3 second timeout with usleep(60000)
             {
                 break;
             }
@@ -487,17 +485,18 @@ int RingBuffer::safe_read(RemoteFile *rf, void *data, unsigned sz)
 
     do 
     {
-        if (requestedblocks > 3)
+        if (requestedbytes > reqsize * 3)
             break;
 
         if (rf->RequestBlock(reqsize))
         {
             hiteof = true;
+            VERBOSE(VB_PLAYBACK, "RequestBlock extends beyond EOF.");
             break;
         }
-        requestedblocks++;
+        requestedbytes += reqsize;
     }
-    while (requestedblocks < 2);
+    while (requestedbytes <= reqsize * 2);
  
     qApp->lock();
     unsigned int available = sock->bytesAvailable();
@@ -540,12 +539,15 @@ int RingBuffer::safe_read(RemoteFile *rf, void *data, unsigned sz)
         tot += ret;
     }
 
-    requestedblocks--;
+    requestedbytes -= tot;
+
+    VERBOSE(VB_FILE, QString ("sz: %1 return: %2 requested: %3 avail: %4")
+            .arg(sz).arg(tot).arg(requestedbytes).arg(available));
 
     return tot;
 }
 
-#define READ_AHEAD_SIZE (10 * 1024 * 1024)
+#define READ_AHEAD_SIZE (10 * 256000)
 
 void RingBuffer::CalcReadAheadThresh(int estbitrate)
 {
@@ -571,7 +573,7 @@ void RingBuffer::CalcReadAheadThresh(int estbitrate)
     if (estbitrate > 17000)
         readblocksize = 256000;
 
-    fill_min = readblocksize;
+    fill_min = 1;
 
     readsallowed = false;
 
@@ -587,14 +589,14 @@ int RingBuffer::ReadBufFree(void)
 {
     int ret = 0;
 
-    pthread_mutex_lock(&readAheadLock);
+    readAheadLock.lock();
 
     if (rbwpos >= rbrpos)
         ret = rbrpos + READ_AHEAD_SIZE - rbwpos - 1;
     else
         ret = rbrpos - rbwpos - 1;
 
-    pthread_mutex_unlock(&readAheadLock);
+    readAheadLock.unlock();
     return ret;
 }
 
@@ -602,36 +604,34 @@ int RingBuffer::ReadBufAvail(void)
 {
     int ret = 0;
     
-    pthread_mutex_lock(&readAheadLock);
+    readAheadLock.lock();
 
     if (rbwpos >= rbrpos)
         ret = rbwpos - rbrpos;
     else
         ret = READ_AHEAD_SIZE - rbrpos + rbwpos;
 
-    pthread_mutex_unlock(&readAheadLock);
-
+    readAheadLock.unlock();
     return ret;
 }
 
 // must call with rwlock in write lock
 void RingBuffer::ResetReadAhead(long long newinternal)
 {
-    pthread_mutex_lock(&readAheadLock);
+    readAheadLock.lock();
     rbrpos = 0;
     rbwpos = 0;
     internalreadpos = newinternal;
     ateof = false;
     readsallowed = false;
-    requestedblocks = 0;
-    pthread_mutex_unlock(&readAheadLock);
+    requestedbytes = 0;
+    readAheadLock.unlock();
 }
 
 void RingBuffer::StartupReadAheadThread(void)
 {
     readaheadrunning = false;
 
-    pthread_mutex_init(&readAheadLock, NULL);
     pthread_create(&reader, NULL, startReader, this);
 
     while (!readaheadrunning)
@@ -647,6 +647,17 @@ void RingBuffer::KillReadAheadThread(void)
     pthread_join(reader, NULL);
 }
 
+void RingBuffer::StopReads(void)
+{
+    stopreads = true;
+    availWait.wakeAll();
+}
+
+void RingBuffer::StartReads(void)
+{
+    stopreads = false;
+}
+
 void RingBuffer::Pause(void)
 {
     pausereadthread = true;
@@ -667,6 +678,15 @@ bool RingBuffer::isPaused(void)
     return readaheadpaused;
 }
 
+void RingBuffer::WaitForPause(void)
+{
+    if  (!readaheadpaused)
+    {
+        while (!pauseWait.wait(1000))
+            VERBOSE(VB_IMPORTANT, "Waited too long for ringbuffer pause..");
+    }
+}
+
 void *RingBuffer::startReader(void *type)
 {
     RingBuffer *rbuffer = (RingBuffer *)type;
@@ -693,6 +713,7 @@ void RingBuffer::ReadAheadThread(void)
         if (pausereadthread)
         {
             readaheadpaused = true;
+            pauseWait.wakeAll();
             usleep(100);
             totfree = ReadBufFree();
             continue;
@@ -765,9 +786,9 @@ void RingBuffer::ReadAheadThread(void)
                 }
             }
 
-            pthread_mutex_lock(&readAheadLock);
+            readAheadLock.lock();
             rbwpos = (rbwpos + ret) % READ_AHEAD_SIZE;
-            pthread_mutex_unlock(&readAheadLock);
+            readAheadLock.unlock();
 
             if (ret == 0 && normalfile && !stopreads)
             {
@@ -790,15 +811,25 @@ void RingBuffer::ReadAheadThread(void)
         if (readsallowed && used < fill_min && !ateof)
         {
             readsallowed = false;
-            cerr << "rebuffering (" << used << " " << fill_min << ")\n";
+            VERBOSE(VB_GENERAL, QString ("rebuffering (%1 %2)").arg(used)
+                                                               .arg(fill_min));
         }
 
+        if (readsallowed || stopreads)
+            readsAllowedWait.wakeAll();
+
+        availWaitMutex.lock();
+        if (ateof || stopreads || (wanttoread <= used && wanttoread > 0))
+            availWait.wakeAll();
+        availWaitMutex.unlock();
+            
         pthread_rwlock_unlock(&rwlock);
 
-        if (used >= fill_threshold || wantseek)
+        if ((used >= fill_threshold || wantseek) && !pausereadthread)
             usleep(500);
 
-        sched_yield();
+        if (!pausereadthread)
+            sched_yield();
     }
 
     delete [] readAheadBuffer;
@@ -818,19 +849,29 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
     }
     else 
         while (!readsallowed && !stopreads)
-            usleep(100);
+        {
+            if (!readsAllowedWait.wait(5000))
+                cerr << "taking too long to be allowed to read..\n";
+        }
 
     int avail = ReadBufAvail();
 
     while (avail < count && !stopreads)
     {
-        usleep(100);
+        availWaitMutex.lock();
+        wanttoread = count;
+        if (!availWait.wait(&availWaitMutex, 2000))
+            cerr << "Waited 2 seconds for data to become available, waiting "
+                    "again...\n";
+        wanttoread = 0;
+        availWaitMutex.unlock();
+
         avail = ReadBufAvail();
         if (ateof && avail < count)
             count = avail;
     }
 
-    if (stopreads && avail < count)
+    if ((ateof || stopreads) && avail < count)
         count = avail;
 
     if (rbrpos + count > READ_AHEAD_SIZE)
@@ -844,15 +885,14 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
     else
         memcpy(buf, readAheadBuffer + rbrpos, count);
 
-    pthread_mutex_lock(&readAheadLock);
+    readAheadLock.lock();
     rbrpos = (rbrpos + count) % READ_AHEAD_SIZE;
-    pthread_mutex_unlock(&readAheadLock);
+    readAheadLock.unlock();
 
     if (readone)
     {
         Pause();
-        while (!isPaused())
-            usleep(50);
+        WaitForPause();
     }
 
     return count;
@@ -905,16 +945,34 @@ int RingBuffer::Read(void *buf, int count)
         }
         else
         {
+            if (stopreads)
+            {
+                pthread_rwlock_unlock(&rwlock);
+                return 0;
+            }
+
+            availWaitMutex.lock();
+            wanttoread = totalreadpos + count;
+
             while (totalreadpos + count > totalwritepos - tfw->BufUsed())
             {
-                usleep(1000);
+                if (!availWait.wait(&availWaitMutex, 15000))
+                {
+                    cerr << "Couldn't read data from the capture card in 15 "
+                            "seconds.  Game over, man.\n";
+                    StopReads();
+                }
+
                 if (stopreads)
                 {
+                    availWaitMutex.unlock();
                     pthread_rwlock_unlock(&rwlock);
                     return 0;
                 }
             }
-            
+
+            availWaitMutex.unlock();
+
             if (readpos + count > filesize)
             {
                 int toread = filesize - readpos;
@@ -1014,6 +1072,11 @@ int RingBuffer::Write(const void *buf, int count)
             writepos += ret;
             totalwritepos += ret;
         }
+
+        availWaitMutex.lock();
+        if (totalwritepos >= wanttoread)
+            availWait.wakeAll();
+        availWaitMutex.unlock();
     }
 
     pthread_rwlock_unlock(&rwlock);
