@@ -8,12 +8,13 @@ using namespace std;
 #include "NuppelVideoPlayer.h"
 #include "remoteencoder.h"
 #include "programinfo.h"
+#include "mythcontext.h"
 
-#ifdef USING_XVMC
 extern "C" {
+#ifdef USING_XVMC
 #include "../libavcodec/xvmc_render.h"
-}
 #endif
+}
 
 extern pthread_mutex_t avcodeclock;
 
@@ -30,6 +31,7 @@ AvFormatDecoder::AvFormatDecoder(NuppelVideoPlayer *parent, QSqlDatabase *db,
     framesPlayed = 0;
     framesRead = 0;
 
+    audio_check_1st = 2;
     audio_sample_size = 4;
     audio_sampling_rate = 48000;
 
@@ -41,7 +43,8 @@ AvFormatDecoder::AvFormatDecoder(NuppelVideoPlayer *parent, QSqlDatabase *db,
 
     exitafterdecoded = false;
     ateof = false;
-    gopset = 0;
+    gopset = false;
+    firstgoppos = 0;
 
     fps = 29.97;
     validvpts = false;
@@ -222,7 +225,6 @@ void AvFormatDecoder::InitByteContext(void)
 int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                               char testbuf[2048])
 {
-    (void)novideo;
     ringBuffer = rbuffer;
 
     AVInputFormat *fmt = NULL;
@@ -302,15 +304,17 @@ int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 enc->debug = 0;
                 enc->rate_emu = 0;
 
-#ifdef USING_XVMC
-                if (enc->codec_id == CODEC_ID_MPEG1VIDEO)
+                if (!novideo && (enc->codec_id == CODEC_ID_MPEG1VIDEO || 
+                    enc->codec_id == CODEC_ID_MPEG2VIDEO))
                 {
+#ifdef USING_XVMC
                     enc->codec_id = CODEC_ID_MPEG2VIDEO_XVMC;
-                    enc->slice_flags = SLICE_FLAG_CODED_ORDER | 
-                                       SLICE_FLAG_ALLOW_FIELD;
-                }
 #endif            
-        
+#ifdef USING_VIASLICE
+                    enc->codec_id = CODEC_ID_MPEG2VIDEO_VIA;
+#endif
+                }
+
                 AVCodec *codec = avcodec_find_decoder(enc->codec_id);
 
                 if (codec && codec->capabilities & CODEC_CAP_TRUNCATED)
@@ -326,6 +330,21 @@ int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                     directrendering = true;
                     enc->slice_flags = SLICE_FLAG_CODED_ORDER |
                                        SLICE_FLAG_ALLOW_FIELD;
+                    setLowBuffers();
+                    m_parent->ForceVideoOutputType(kVideoOutput_XvMC);
+                }
+		else if (codec && codec->id == CODEC_ID_MPEG2VIDEO_VIA)
+                {
+                    enc->flags |= CODEC_FLAG_EMU_EDGE;
+                    enc->get_buffer = get_avf_buffer_via;
+                    enc->release_buffer = release_avf_buffer_via;
+                    enc->draw_horiz_band = render_slice_via;
+                    enc->opaque = (void *)this;
+                    directrendering = true;
+                    enc->slice_flags = SLICE_FLAG_CODED_ORDER |
+                                       SLICE_FLAG_ALLOW_FIELD;
+                    setLowBuffers();
+                    m_parent->ForceVideoOutputType(kVideoOutput_VIA);
                 }
                 else if (codec && codec->capabilities & CODEC_CAP_DR1 && 
                     !(enc->width % 16))
@@ -347,6 +366,7 @@ int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 m_parent->SetAudioParams(16, enc->channels, enc->sample_rate);
                 audio_sample_size = enc->channels * 2;
                 audio_sampling_rate = enc->sample_rate;
+                audio_channels = enc->channels;
                 break;
             }
             default:
@@ -399,7 +419,7 @@ int AvFormatDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
 
     dump_format(ic, 0, filename, 0);
     if (hasFullPositionMap)
-        cout << "Position map found\n";
+        VERBOSE(VB_PLAYBACK, "Position map found");
 
     return hasFullPositionMap;
 }
@@ -431,6 +451,64 @@ bool AvFormatDecoder::CheckVideoParams(int width, int height)
     return true;
 }
 
+bool AvFormatDecoder::CheckAudioParams(int freq, int channels)
+{
+    if (audio_check_1st == 2)
+    {
+        if (freq == audio_sampling_rate && channels == audio_channels)
+            return false;
+
+        audio_check_1st = 1;
+        audio_sampling_rate_2nd = freq;
+        audio_channels_2nd = channels;
+        return false;
+    }
+    else
+    {
+        if ((freq != audio_sampling_rate_2nd && channels != audio_channels_2nd)
+            || (freq == audio_sampling_rate && channels == audio_channels))
+        {
+            audio_sampling_rate_2nd = -1;
+            audio_channels_2nd = -1;
+            audio_check_1st = 2;
+            return false;
+        }
+
+        if (audio_check_1st == 1)
+        {
+            audio_check_1st = 0;
+            return false;
+        }
+
+        audio_check_1st = 2;
+    }
+
+    QString chan = "stereo";
+    if (channels == 1)
+        chan = "mono";
+
+    cerr << "Audio has changed: " << freq << "hz " << chan << endl;
+
+    for (int i = 0; i < ic->nb_streams; i++)
+    {
+        AVCodecContext *enc = &ic->streams[i]->codec;
+        switch (enc->codec_type)
+        {
+            case CODEC_TYPE_AUDIO:
+            {
+                AVCodec *codec = enc->codec;
+                avcodec_close(enc);
+                avcodec_open(enc, codec);
+                break;
+            }
+            default:
+                break;
+        }
+    }
+
+    return true;
+}
+
 int get_avf_buffer(struct AVCodecContext *c, AVFrame *pic)
 {
     AvFormatDecoder *nd = (AvFormatDecoder *)(c->opaque);
@@ -540,6 +618,55 @@ void render_slice_xvmc(struct AVCodecContext *s, AVFrame *src, int offset[4],
     nd->m_parent->DrawSlice(frame, 0, y, width, height);
 }
 
+int get_avf_buffer_via(struct AVCodecContext *c, AVFrame *pic)
+{
+    AvFormatDecoder *nd = (AvFormatDecoder *)(c->opaque);
+
+    VideoFrame *frame = nd->m_parent->GetNextVideoFrame();
+
+    pic->data[0] = frame->buf;
+    pic->data[1] = NULL;
+    pic->data[2] = NULL;
+
+    pic->linesize[0] = 0;
+    pic->linesize[1] = 0;
+    pic->linesize[2] = 0;
+
+    pic->opaque = frame;
+    pic->type = FF_BUFFER_TYPE_USER;
+
+    pic->age = 256 * 256 * 256 * 64;
+
+    nd->inUseBuffers.append(frame);
+
+    return 1;
+}
+
+void release_avf_buffer_via(struct AVCodecContext *c, AVFrame *pic)
+{
+    (void)c;
+
+    assert(pic->type == FF_BUFFER_TYPE_USER);
+
+    int i;
+    for (i = 0; i < 4; i++)
+        pic->data[i] = NULL;
+}
+
+void render_slice_via(struct AVCodecContext *s, AVFrame *src, int offset[4],
+                      int y, int type, int height)
+{
+    (void)offset;
+    (void)type;
+
+    AvFormatDecoder *nd = (AvFormatDecoder *)(s->opaque);
+
+    int width = s->width;
+
+    VideoFrame *frame = (VideoFrame *)src->opaque;
+    nd->m_parent->DrawSlice(frame, 0, y, width, height);
+}
+
 #define SEQ_START     0x000001b3
 #define GOP_START     0x000001b8
 #define PICTURE_START 0x00000100
@@ -582,7 +709,6 @@ static const float avfmpeg2_aspect[16]={
     0,    1.0,    -3.0/4.0,    -9.0/16.0,    -1.0/2.21,
 };
 
-
 float AvFormatDecoder::GetMpegAspect(AVCodecContext *context,
                                      int aspect_ratio_info,
                                      int width, int height)
@@ -691,7 +817,9 @@ void AvFormatDecoder::GetFrame(int onlyvideo)
         {
             AVCodecContext *context = &(curstream->codec);
             if (context->codec_id == CODEC_ID_MPEG1VIDEO ||
-                context->codec_id == CODEC_ID_MPEG2VIDEO_XVMC)
+                context->codec_id == CODEC_ID_MPEG2VIDEO ||
+                context->codec_id == CODEC_ID_MPEG2VIDEO_XVMC ||
+                context->codec_id == CODEC_ID_MPEG2VIDEO_VIA)
             {
                 int startpos = 0;
                 if ((startpos = PacketHasHeader(ptr, len, SEQ_START)))
@@ -710,7 +838,7 @@ void AvFormatDecoder::GetFrame(int onlyvideo)
                     {
                         m_parent->SetVideoParams(width, height, fps,
                                                  keyframedist, aspect);
-                        m_parent->Reinit();
+                        m_parent->ReinitVideo();
                         current_width = width;
                         current_height = height;
                         current_aspect = aspect;
@@ -730,9 +858,14 @@ void AvFormatDecoder::GetFrame(int onlyvideo)
  
                     if (!gopset && frameNum > 0)
                     {
-                        keyframedist = frameNum;
-                        gopset = true;
-                        m_parent->SetVideoParams(-1, -1, -1, keyframedist);
+                        if (firstgoppos > 0)
+                        {
+                            keyframedist = frameNum - firstgoppos;
+                            gopset = true;
+                            m_parent->SetVideoParams(-1, -1, -1, keyframedist);
+                        }
+                        else
+                            firstgoppos = frameNum;
                     }
 
                     lastKey = frameNum;
@@ -764,6 +897,19 @@ void AvFormatDecoder::GetFrame(int onlyvideo)
                         continue;
                     }
 
+                    if (CheckAudioParams(curstream->codec.sample_rate,
+                                         curstream->codec.channels))
+                    {
+                        audio_sampling_rate = curstream->codec.sample_rate;
+                        audio_channels = curstream->codec.channels;
+                        audio_sample_size = audio_channels * 2;
+
+                        m_parent->SetEffDsp(audio_sampling_rate * 100);
+                        m_parent->SetAudioParams(16, audio_channels,
+                                                 audio_sampling_rate);
+                        m_parent->ReinitAudio();
+                    }
+
                     temppts = (long long)((double)pkt->pts * ptsmultiplier); 
 
                     if (lastapts != temppts && temppts > 0 && validvpts)
@@ -845,12 +991,6 @@ void AvFormatDecoder::GetFrame(int onlyvideo)
                             newvpts = (long long)((double)pkt->pts * 
                                                   ptsmultiplier); 
                         }
-                        else if (context->codec_id == CODEC_ID_MPEG1VIDEO)
-                        {
-                            // guess, based off of the audio timestamp and 
-                            // the prebuffer size
-                            newvpts = lastapts + (int)(1000.0 / fps) * 3;
-                        }
 
                         if (newvpts <= lastvpts)
                             lastvpts += (int)(1000.0 / fps);
@@ -922,9 +1062,34 @@ bool AvFormatDecoder::DoRewind(long long desiredFrame)
         }
     }
 
+    int iter = 0;
+
+    while (keyPos == 0 && lastKey > 1 && iter < 4)
+    {
+        VERBOSE(VB_PLAYBACK, QString("No keyframe in position map for %1")
+                                     .arg((int)lastKey));
+
+        lastKey -= keyframedist;
+        keyPos = positionMap[lastKey / keyframedist];
+        diff = keyPos - curPosition;
+        normalframes += keyframedist;
+
+        if (keyPos != 0)
+           VERBOSE(VB_PLAYBACK, QString("Using seek position: %1")
+                                       .arg((int)lastKey));
+
+        iter++;
+    }
+
     if (keyPos == 0)
     {
-        cout << "unknown position: " << lastKey << endl;
+        VERBOSE(VB_GENERAL, QString("Unknown seek position: %1")
+                                    .arg((int)lastKey));
+
+        lastKey = storelastKey;
+        diff = 0;
+        normalframes = 0;
+
         return false;
     }
 
@@ -969,23 +1134,36 @@ bool AvFormatDecoder::DoFastForward(long long desiredFrame)
 
     long long keyPos = -1;
 
-    if (desiredKey != lastKey)
-    {
-        int desiredIndex = desiredKey / keyframedist;
+    long long tmpKey = desiredKey;
+    int tmpIndex = tmpKey / keyframedist;
 
-        if (positionMap.find(desiredIndex) != positionMap.end())
+    while (keyPos == -1 && tmpKey > lastKey)
+    {
+        if (positionMap.find(tmpIndex) != positionMap.end())
         {
-            keyPos = positionMap[desiredIndex];
+            keyPos = positionMap[tmpIndex];
         }
         else if (livetv || (watchingrecording && nvr_enc &&
                             nvr_enc->IsValidRecorder()))
         {
-            for (int i = lastKey / keyframedist; i <= desiredIndex; i++)
+            for (int i = lastKey / keyframedist; i <= tmpIndex; i++)
             {
                 if (positionMap.find(i) == positionMap.end())
-                    nvr_enc->FillPositionMap(i, desiredIndex, positionMap);
+                    nvr_enc->FillPositionMap(i, tmpIndex, positionMap);
             }
-            keyPos = positionMap[desiredIndex];
+            keyPos = positionMap[tmpIndex];
+        }
+        if (keyPos == -1)
+        {
+            VERBOSE(VB_PLAYBACK, QString("No keyframe in position map for %1")
+                    .arg((int)tmpKey));
+
+            tmpKey -= keyframedist;
+            tmpIndex--;
+        }
+        else
+        {
+            desiredKey = tmpKey;
         }
     }
 
@@ -1004,6 +1182,11 @@ bool AvFormatDecoder::DoFastForward(long long desiredFrame)
     }
     else if (desiredKey != lastKey && !livetv && !watchingrecording)
     {
+        VERBOSE(VB_IMPORTANT, "Did not find keyframe position.");
+
+        VERBOSE(VB_PLAYBACK, QString("lastKey: %1 desiredKey: %2")
+                .arg((int)lastKey).arg((int)desiredKey));
+
         while (framesRead < desiredKey + 1 || 
                !positionMap.contains(desiredKey / keyframedist))
         {
