@@ -60,6 +60,11 @@ class AvFormatDecoder : public DecoderBase
     friend void render_slice_xvmc(struct AVCodecContext *c, AVFrame *src, 
                                   int offset[4], int y, int type, int height);
 
+    friend int get_avf_buffer_via(struct AVCodecContext *c, AVFrame *pic);
+    friend void release_avf_buffer_via(struct AVCodecContext *c, AVFrame *pic);
+    friend void render_slice_via(struct AVCodecContext *c, AVFrame *src,
+                                 int offset[4], int y, int type, int height);
+	
     friend int open_avf(URLContext *h, const char *filename, int flags);
     friend int read_avf(URLContext *h, uint8_t *buf, int buf_size);
     friend int write_avf(URLContext *h, uint8_t *buf, int buf_size);
@@ -74,6 +79,7 @@ class AvFormatDecoder : public DecoderBase
     void SeekReset(void);
 
     bool CheckVideoParams(int width, int height);
+    bool CheckAudioParams(int freq, int channels);
 
     RingBuffer *ringBuffer;
 
@@ -91,6 +97,11 @@ class AvFormatDecoder : public DecoderBase
 
     int audio_sample_size;
     int audio_sampling_rate;
+    int audio_channels;
+
+    int audio_check_1st;
+    int audio_sampling_rate_2nd;
+    int audio_channels_2nd;
 
     long long lastapts;
     long long lastvpts;
@@ -126,6 +137,8 @@ class AvFormatDecoder : public DecoderBase
     float current_aspect;
 
     QPtrList<AVPacket> storedPackets;
+
+    int firstgoppos;
 };
 
 #endif
