@@ -77,6 +77,7 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     DISPLAY_TIMES = 30;
     int maxchannel = 0;
     m_currentStartChannel = 0;
+    ignoreevents = false;
 
     m_player = player;
     m_db = QSqlDatabase::database();
@@ -211,10 +212,15 @@ GuideGrid::GuideGrid(MythMainWindow *parent, const QString &channel, TV *player,
     updateBackground();
 
     setNoErase();
+    gContext->addListener(this);
+    
+    keyDown = false;
+    setFocusPolicy(QWidget::ClickFocus); //get keyRelease events after refocus
 }
 
 GuideGrid::~GuideGrid()
 {
+    gContext->removeListener(this);
     for (int x = 0; x < DISPLAY_TIMES; x++)
     {
         if (m_timeInfos[x])
@@ -234,6 +240,20 @@ GuideGrid::~GuideGrid()
 
 void GuideGrid::keyPressEvent(QKeyEvent *e)
 {
+    //keyDown limits keyrepeats to prevent continued scrolling
+    //after key is released. Note: Qt's keycompress should handle
+    //this but will fail with fast key strokes and keyboard repeat
+    //enabled. Keys will not be marked as autorepeat and flood buffer.
+    //setFocusPolicy(QWidget::ClickFocus) in constructor is important 
+    //or keyRelease events will not be received after a refocus.
+    
+    if(keyDown && e->key() != Key_Escape && e->key() != Key_C)
+        return; //check for Key_Escape incase something goes wrong 
+                //with KeyRelease events, shouldn't happen.
+       
+    if(e->key() != Key_Control)
+        keyDown = true;    
+    
     if (e->state() == Qt::ControlButton)
     {
         switch (e->key())
@@ -276,6 +296,14 @@ void GuideGrid::keyPressEvent(QKeyEvent *e)
     }
 }
 
+void GuideGrid::keyReleaseEvent(QKeyEvent *e)
+{
+    //note: KeyRelease events may not reflect the released 
+    //key if delayed (key==0 instead).
+    e->key(); //stop compiler warning
+    keyDown = false;
+}
+
 void GuideGrid::updateBackground(void)
 {
     QPixmap bground(size());
@@ -440,9 +468,9 @@ void GuideGrid::fillChannelInfos(int &maxchannel, bool gotostartchannel)
                     val.callsign = "";
                 val.iconpath = query.value(2).toString();
                 if (val.iconpath == QString::null)
-                    val.iconpath = "(null)";
+                    val.iconpath = "none";
                 if (val.iconpath.stripWhiteSpace().length() == 0)
-                    val.iconpath = "(null)";
+                    val.iconpath = "none";
                 val.chanstr = query.value(0).toString();
                 if (val.chanstr == QString::null)
                 val.chanstr = "";
@@ -748,6 +776,24 @@ void GuideGrid::fillProgramRowInfos(unsigned int row)
     } 
 }
 
+void GuideGrid::customEvent(QCustomEvent *e)
+{
+    if (ignoreevents)
+        return;
+
+    if ((MythEvent::Type)(e->type()) == MythEvent::MythEventMessage)
+    {
+        MythEvent *me = (MythEvent *)e;
+        QString message = me->Message();
+
+        if (message == "SCHEDULE_CHANGE")
+        {
+            fillProgramInfos();
+            update(fullRect);
+        }
+    }
+}
+
 void GuideGrid::paintEvent(QPaintEvent *e)
 {
     QRect r = e->rect();
@@ -755,6 +801,7 @@ void GuideGrid::paintEvent(QPaintEvent *e)
 
     if (!showInfo)
     {
+        ignoreevents = true;
         if (r.intersects(infoRect))
             paintInfo(&p);
         if (r.intersects(dateRect))
@@ -767,6 +814,7 @@ void GuideGrid::paintEvent(QPaintEvent *e)
             paintPrograms(&p);
         if (r.intersects(curInfoRect))
             paintCurrentInfo(&p);
+        ignoreevents = false;
     }
 }
 
@@ -1147,9 +1195,9 @@ void GuideGrid::cursorLeft()
     else
     {
         fillProgramRowInfos(m_currentRow);
-        update(programRect);
-        update(infoRect);
-        update(timeRect);
+        repaint(programRect, false);
+        repaint(infoRect, false);
+        repaint(timeRect, false);
     }
 }
 
@@ -1176,9 +1224,9 @@ void GuideGrid::cursorRight()
     else
     {
         fillProgramRowInfos(m_currentRow);
-        update(programRect);
-        update(infoRect);
-        update(timeRect);
+        repaint(programRect, false);
+        repaint(infoRect, false);
+        repaint(timeRect, false); 
     }
 }
 
@@ -1196,8 +1244,8 @@ void GuideGrid::cursorDown()
         else
         {
             fillProgramRowInfos(m_currentRow);
-            update(programRect);
-            update(infoRect);
+            repaint(programRect, false);
+            repaint(infoRect, false);
         }
     }
     else
@@ -1218,8 +1266,8 @@ void GuideGrid::cursorUp()
         else
         {
             fillProgramRowInfos(m_currentRow);
-            update(programRect);
-            update(infoRect);
+            repaint(programRect, false);
+            repaint(infoRect, false);
         }
     }
     else
@@ -1242,10 +1290,10 @@ void GuideGrid::scrollLeft()
     fillTimeInfos();
     fillProgramInfos();
 
-    update(infoRect);
-    update(timeRect);
-    update(dateRect);
-    update(programRect);
+    repaint(programRect, false);
+    repaint(infoRect, false);
+    repaint(dateRect, false);
+    repaint(timeRect, false);
 }
 
 void GuideGrid::scrollRight()
@@ -1263,10 +1311,10 @@ void GuideGrid::scrollRight()
     fillTimeInfos();
     fillProgramInfos();
 
-    update(infoRect);
-    update(dateRect);
-    update(timeRect);
-    update(programRect);
+    repaint(programRect, false);
+    repaint(infoRect, false);
+    repaint(dateRect, false);
+    repaint(timeRect, false);
 }
 
 void GuideGrid::setStartChannel(int newStartChannel)
@@ -1295,9 +1343,9 @@ void GuideGrid::scrollDown()
     m_programs[DISPLAY_CHANS - 1] = proglist;
     fillProgramInfos();
 
-    update(infoRect);
-    update(channelRect);
-    update(programRect);
+    repaint(programRect, false);
+    repaint(infoRect, false);
+    repaint(channelRect, false);
 }
 
 void GuideGrid::scrollUp()
@@ -1316,9 +1364,9 @@ void GuideGrid::scrollUp()
     m_programs[0] = proglist;
     fillProgramInfos();
 
-    update(infoRect);
-    update(channelRect);
-    update(programRect);
+    repaint(programRect, false);
+    repaint(infoRect, false);
+    repaint(channelRect, false);
 }
 
 void GuideGrid::dayLeft()
@@ -1328,7 +1376,7 @@ void GuideGrid::dayLeft()
     fillTimeInfos();
     fillProgramInfos();
 
-    update(fullRect);
+    repaint(fullRect, false);
 }
 
 void GuideGrid::dayRight()
@@ -1338,7 +1386,7 @@ void GuideGrid::dayRight()
     fillTimeInfos();
     fillProgramInfos();
 
-    update(fullRect);
+    repaint(fullRect, false);
 }
 
 void GuideGrid::pageLeft()
@@ -1357,7 +1405,7 @@ void GuideGrid::pageLeft()
     fillTimeInfos();
     fillProgramInfos();
 
-    update(fullRect);
+    repaint(fullRect, false);
 }
 
 void GuideGrid::pageRight()
@@ -1376,7 +1424,7 @@ void GuideGrid::pageRight()
     fillTimeInfos();
     fillProgramInfos();
 
-    update(fullRect);
+    repaint(fullRect, false);
 }
 
 void GuideGrid::pageDown()
@@ -1385,7 +1433,7 @@ void GuideGrid::pageDown()
 
     fillProgramInfos();
 
-    update(fullRect);
+    repaint(fullRect, false);
 }
 
 void GuideGrid::pageUp()
@@ -1394,7 +1442,7 @@ void GuideGrid::pageUp()
 
     fillProgramInfos();
 
-    update(fullRect);
+    repaint(fullRect, false);
 }
  
 void GuideGrid::showProgFinder()
@@ -1471,6 +1519,8 @@ void GuideGrid::displayInfo()
 
     if (pginfo)
     {
+        FocusPolicy storeFocus = focusPolicy();
+        setFocusPolicy(QWidget::NoFocus);
         if ((gContext->GetNumSetting("AdvancedRecord", 0)) ||
             (pginfo->GetProgramRecordingStatus(m_db) > kAllRecord))
         {
@@ -1483,6 +1533,7 @@ void GuideGrid::displayInfo()
             InfoDialog diag(pginfo, gContext->GetMainWindow(), "Program Info");
             diag.exec();
         }
+        setFocusPolicy(storeFocus);
     }
     else
         return;
