@@ -25,6 +25,7 @@ class HDTVRecorder : public RecorderBase
     void Pause(bool clear = true);
     void Unpause(void);
     bool GetPause(void);
+    void WaitForPause(void);
     bool IsRecording(void);
 
     long long GetFramesWritten(void);
@@ -38,9 +39,14 @@ class HDTVRecorder : public RecorderBase
     bool SetupRecording();
     void FinishRecording();
 
-    bool PacketHasHeader(unsigned char *buf, int len, unsigned int startcode);
-    void ProcessData(unsigned char *buffer, int len);
+    int ProcessData(unsigned char *buffer, int len);
+    void FindKeyframes(const unsigned char *buffer, 
+		       int packet_start_pos,
+		       int pkt_pos,
+		       char adaptation_field_control,
+		       bool payload_unit_start_indicator);
 
+    int ResyncStream(unsigned char *buffer, int curr_pos, int len);
     bool recording;
     bool encoding;
 
@@ -53,9 +59,6 @@ class HDTVRecorder : public RecorderBase
     int width, height;
 
     int chanfd;
-    int readfd;
-
-    AVFormatContext *ic;
 
     int keyframedist;
     bool gopset;
@@ -63,6 +66,18 @@ class HDTVRecorder : public RecorderBase
     QMap<long long, long long> positionMap;
 
     long long prev_gop_save_pos;
+    int firstgoppos;
+
+    int pat_pid;
+    int psip_pid;
+    int video_pid;
+    int audio_pid;
+    int base_pid;
+
+    int ts_packets;
+
+    int lowest_video_pid;
+    int video_pid_packets;
 };
 
 #endif
