@@ -10,6 +10,7 @@ using namespace std;
 #include "nuppeldecoder.h"
 #include "NuppelVideoPlayer.h"
 #include "remoteencoder.h"
+#include "mythcontext.h"
 
 #include "minilzo.h"
 
@@ -418,10 +419,17 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
         foundit = (FRAMEHEADERSIZE != ringBuffer->Read(&frameheader,
                                                        FRAMEHEADERSIZE));
 
+        bool framesearch = false;
+
         while (frameheader.frametype != 'A' && frameheader.frametype != 'V' &&
                frameheader.frametype != 'S' && frameheader.frametype != 'T' &&
                frameheader.frametype != 'R' && frameheader.frametype != 'X')
         {
+            if (!framesearch)
+                VERBOSE(VB_IMPORTANT, "Searching for frame header.");
+
+            framesearch = true;
+
             ringBuffer->Seek(startpos2, SEEK_SET);
 
             char dummychar;
@@ -706,7 +714,8 @@ bool NuppelDecoder::DecodeFrame(struct rtframeheader *frameheader,
         pthread_mutex_unlock(&avcodeclock);
         if (ret < 0)
         {
-            cout << "decoding error: " << ret << " back from avcodec\n";
+            VERBOSE(VB_PLAYBACK, QString("decoding error: %1 back from avcodec")
+                                 .arg(ret));
             return false;
         }
 
@@ -810,8 +819,15 @@ void NuppelDecoder::GetFrame(int avignore)
             return;
         }
 
+        bool framesearch = false;
+
         while (!isValidFrametype(frameheader.frametype))
         {
+            if (!framesearch)
+                VERBOSE(VB_IMPORTANT, "Searching for frame header.");
+
+            framesearch = true;
+
             ringBuffer->Seek((long long)seeklen-FRAMEHEADERSIZE, SEEK_CUR);
 
             if (ringBuffer->Read(&frameheader, FRAMEHEADERSIZE)
@@ -891,6 +907,7 @@ void NuppelDecoder::GetFrame(int avignore)
             if (avignore == -1)
             {
                 framesPlayed++;
+                gotvideo = 1;
                 continue;
             }
 
@@ -1062,9 +1079,35 @@ bool NuppelDecoder::DoRewind(long long desiredFrame)
         }
     }
 
+    int iter = 0;
+
+    while (keyPos == 0 && lastKey > 1 && iter < 4)
+    {
+        VERBOSE(VB_PLAYBACK, QString("No keyframe in position map for %1")
+                                     .arg((int)lastKey));
+
+        lastKey -= keyframedist;
+        keyIndex--;
+        keyPos = (*positionMap)[keyIndex];
+        diff = keyPos - curPosition;
+        normalframes += keyframedist;
+
+        if (keyPos != 0)
+           VERBOSE(VB_PLAYBACK, QString("Using seek position: %1")
+                                       .arg((int)lastKey));
+
+        iter++;
+    }
+
     if (keyPos == 0)
     {
-        cout << "unknown position: " << lastKey << endl;
+        VERBOSE(VB_GENERAL, QString("Unknown seek position: %1")
+                                    .arg((int)lastKey));
+
+        lastKey = storelastKey;
+        diff = 0;
+        normalframes = 0;
+
         return false;
     }
 
@@ -1152,21 +1195,38 @@ bool NuppelDecoder::DoFastForward(long long desiredFrame)
 
     long long keyPos = -1;
 
-    if (desiredKey != lastKey)
+    long long tmpKey = desiredKey;
+    int tmpIndex = desiredIndex;
+
+    while (keyPos == -1 && tmpKey > lastKey)
     {
-        if (positionMap->find(desiredIndex) != positionMap->end())
+        if (positionMap->find(tmpIndex) != positionMap->end())
         {
-            keyPos = (*positionMap)[desiredIndex];
+            keyPos = (*positionMap)[tmpIndex];
         }
         else if (livetv || (watchingrecording && nvr_enc &&
                             nvr_enc->IsValidRecorder()))
         {
-            for (int i = lastKeyIndex; i <= desiredIndex; i++)
+            for (int i = lastKeyIndex; i <= tmpIndex; i++)
             {
                 if (positionMap->find(i) == positionMap->end())
-                    nvr_enc->FillPositionMap(i, desiredIndex, *positionMap);
+                    nvr_enc->FillPositionMap(i, tmpIndex, *positionMap);
             }
-            keyPos = (*positionMap)[desiredIndex];
+            keyPos = (*positionMap)[tmpIndex];
+        }
+
+        if (keyPos == -1)
+        {
+            VERBOSE(VB_PLAYBACK, QString("No keyframe in position map for %1")
+                    .arg((int)tmpKey));
+
+            tmpKey -= keyframedist;
+            tmpIndex--;
+        }
+        else
+        {
+            desiredKey = tmpKey;
+            desiredIndex = tmpIndex;
         }
     }
 
@@ -1183,12 +1243,31 @@ bool NuppelDecoder::DoFastForward(long long desiredFrame)
     }
     else if (!livetv && !watchingrecording)
     {
+        if (lastKey < desiredKey)
+        {
+            VERBOSE(VB_IMPORTANT, "Did not find keyframe position.");
+
+            VERBOSE(VB_PLAYBACK, QString("lastKey: %1 desiredKey: %2")
+                    .arg((int)lastKey).arg((int)desiredKey));
+        }
+
         while (lastKey < desiredKey && !fileend)
         {
             GetFrame(-1);
             needflush = true;
             fileend = m_parent->GetEof();
         }
+
+        if (needflush)
+        {
+            lastKey = desiredKey;
+            keyPos = (*positionMap)[desiredKey / keyframedist];
+            long long diff = keyPos - ringBuffer->GetTotalReadPosition();
+
+            ringBuffer->Seek(diff, SEEK_CUR);
+
+            framesPlayed = lastKey - 1;
+        }
     }
 
     normalframes = desiredFrame - framesPlayed;
