@@ -17,17 +17,19 @@ using namespace std;
 #include "ttfont.h"
 #include "osd.h"
 #include "osdtypes.h"
+#include "osdsurface.h"
+#include "mythcontext.h"
 #include "libmyth/oldsettings.h"
 
-OSD::OSD(int width, int height, int framerate, const QString &font, 
+OSD::OSD(int width, int height, int frint, const QString &font, 
          const QString &ccfont, const QString &prefix, const QString &osdtheme,
          int dispx, int dispy, int dispw, int disph)
+   : QObject()
 {
-    pthread_mutex_init(&osdlock, NULL);
-
+    changed = false;
     vid_width = width;
     vid_height = height;
-    fps = framerate;
+    frameint = frint;
 
     editarrowleft = editarrowright = NULL;
 
@@ -40,7 +42,7 @@ OSD::OSD(int width, int height, int framerate, const QString &font,
     displayheight = disph;
 
     timeType = 0; 
-    totalfadeframes = 0;
+    totalfadetime = 0;
     fontname = font;
     ccfontname = ccfont;
     fontprefix = prefix;
@@ -67,6 +69,8 @@ OSD::OSD(int width, int height, int framerate, const QString &font,
     }
 
     SetDefaults();
+
+    drawSurface = new OSDSurface(width, height);
 }
 
 OSD::~OSD(void)
@@ -93,6 +97,20 @@ OSD::~OSD(void)
         delete editarrowright;
 
     delete setList;
+    delete drawSurface;
+}
+
+void OSD::SetFrameInterval(int frint)
+{
+    frameint = frint;
+
+    QMap<QString, OSDSet *>::iterator sets = setMap.begin();
+    for (; sets != setMap.end(); ++sets)
+    {
+        OSDSet *set = (*sets);
+        if (set)
+           set->SetFrameInterval(frameint);
+    }
 }
 
 void OSD::SetDefaults(void)
@@ -117,7 +135,8 @@ void OSD::SetDefaults(void)
     if (!container)
     {
         QString name = "cc_page";
-        container = new OSDSet(name, true, vid_width, vid_height, wmult, hmult);
+        container = new OSDSet(name, true, vid_width, vid_height, wmult, hmult,
+                               frameint);
         AddSet(container, name);
 
         OSDTypeCC *ccpage = new OSDTypeCC(name, ccfont, xoffset, yoffset,
@@ -126,13 +145,13 @@ void OSD::SetDefaults(void)
     }
 }
 
-void OSD::Reinit(int width, int height, int fps, int dispx, int dispy, 
+void OSD::Reinit(int width, int height, int frint, int dispx, int dispy, 
                  int dispw, int disph)
 {
     vid_width = width;
     vid_height = height;
-    if (fps != -1)
-       this->fps = fps;
+    if (frint != -1)
+        frameint = frint;
 
     wmult = dispw / 640.0;
     hmult = disph / 480.0;
@@ -156,8 +175,11 @@ void OSD::Reinit(int width, int height, int fps, int dispx, int dispy,
         OSDSet *set = (*sets);
         if (set)
             set->Reinit(vid_width, vid_height, dispx, dispy, dispw, disph, 
-                        wmult, hmult);
+                        wmult, hmult, frameint);
     }
+
+    delete drawSurface;
+    drawSurface = new OSDSurface(displaywidth, displayheight);
 }
 
 QString OSD::FindTheme(QString name)
@@ -748,7 +770,8 @@ void OSD::parseContainer(QDomElement &element)
         exit(0);
     }
 
-    container = new OSDSet(name, true, vid_width, vid_height, wmult, hmult);
+    container = new OSDSet(name, true, vid_width, vid_height, wmult, hmult,
+                           frameint);
 
     QString prio = element.attribute("priority", "");
     if (!prio.isNull() && !prio.isEmpty())
@@ -827,10 +850,10 @@ bool OSD::LoadTheme(void)
     int errorColumn = 0;
 
     if (!doc.setContent(&f, false, &errorMsg, &errorLine, &errorColumn))
-    {
-        cout << "Error parsing: " << themefile << endl;
-        cout << "at line: " << errorLine << "  column: " << errorColumn << endl;
-        cout << errorMsg << endl;
+    { 
+        cerr << "Error parsing: " << themefile << endl;
+        cerr << "at line: " << errorLine << "  column: " << errorColumn << endl;
+        cerr << errorMsg << endl;
         f.close();
         return false;
     }
@@ -850,7 +873,7 @@ bool OSD::LoadTheme(void)
             }
             else if (e.tagName() == "fadeaway")
             {
-                totalfadeframes = getFirstText(e).toInt();
+                totalfadetime = (getFirstText(e).toInt() * 1000000) / 30;
             }
             else if (e.tagName() == "font")
             {
@@ -901,24 +924,24 @@ QRect OSD::parseRect(QString text)
 
 void OSD::ClearAllText(const QString &name)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet(name);
     if (container)
         container->ClearAllText();
 
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::SetTextByRegexp(const QString &name,
                           QMap<QString, QString> &regexpMap, int length)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet(name);
     if (container)
     {
         container->SetTextByRegexp(regexpMap);
         if (length >= 0)
-            container->DisplayFor(length * fps);
+            container->DisplayFor(length * 1000000);
         else
             container->Display();
 
@@ -931,13 +954,14 @@ void OSD::SetTextByRegexp(const QString &name,
         }
 
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::SetInfoText(QMap<QString, QString> regexpMap, int length)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("program_info");
     if (container)
     {
@@ -949,10 +973,11 @@ void OSD::SetInfoText(QMap<QString, QString> regexpMap, int length)
             (regexpMap["iconpath"] != ""))
             cs->LoadImage(regexpMap["iconpath"], wmult, hmult, 30, 30);
 
-        container->DisplayFor(length * fps);
+        container->DisplayFor(length * 1000000);
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::SetInfoText(const QString &text, const QString &subtitle,
@@ -967,7 +992,7 @@ void OSD::SetInfoText(const QString &text, const QString &subtitle,
 
     bool hassubtitle = true;
 
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("program_info");
     if (container)
     {
@@ -1001,18 +1026,29 @@ void OSD::SetInfoText(const QString &text, const QString &subtitle,
         if (cs)
             cs->LoadImage(iconpath, wmult, hmult, 30, 30);
 
-        container->DisplayFor(length * fps);
+        container->DisplayFor(length * 1000000);
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
+}
+
+void OSD::SetUpOSDClosedHandler(TV *tv)
+{
+    OSDSet *container = GetSet("status");
+
+    if (container)
+        connect((QObject *)container, SIGNAL(OSDClosed(int)), (QObject *)tv,
+                SLOT(HandleOSDClosed(int)));
+
 }
 
 void OSD::StartPause(int position, bool fill, QString msgtext,
-                     QString slidertext, int displaytime)
+                     QString slidertext, int displaytime, int osdFunctionalType)
 {
     fill = fill;
 
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("status");
     if (container)
     {
@@ -1028,17 +1064,18 @@ void OSD::StartPause(int position, bool fill, QString msgtext,
             slider->SetPosition(position);
 
         if (displaytime > 0)
-            container->DisplayFor(displaytime * fps);
+            container->DisplayFor(displaytime * 1000000, osdFunctionalType);
         else
             container->Display();
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::UpdatePause(int position, QString slidertext)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("status");
     if (container)
     {
@@ -1051,25 +1088,27 @@ void OSD::UpdatePause(int position, QString slidertext)
             slider->SetPosition(position);
 
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::EndPause(void)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("status");
     if (container)
     {
         container->Hide();
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::SetChannumText(const QString &text, int length)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("channel_number");
     if (container)
     {
@@ -1077,17 +1116,18 @@ void OSD::SetChannumText(const QString &text, int length)
         if (type)
             type->SetText(text);
 
-        container->DisplayFor(length * fps);
+        container->DisplayFor(length * 1000000);
         m_setsvisible = true;
+        changed = true;
     }
 
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::AddCCText(const QString &text, int x, int y, int color, 
                     bool teletextmode)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("cc_page");
     if (container)
     {
@@ -1097,13 +1137,14 @@ void OSD::AddCCText(const QString &text, int x, int y, int color,
 
         container->Display();
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::ClearAllCCText()
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("cc_page");
     if (container)
     {
@@ -1115,13 +1156,14 @@ void OSD::ClearAllCCText()
 
         container->Display(false);
         m_setsvisible = true;
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::SetSettingsText(const QString &text, int length)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet("settings");
     if (container)
     {
@@ -1129,17 +1171,18 @@ void OSD::SetSettingsText(const QString &text, int length)
         if (type)
             type->SetText(text);
 
-        container->DisplayFor(length * fps);
+        container->DisplayFor(length * 1000000);
         m_setsvisible = true;
+        changed = true;
     }
 
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::NewDialogBox(const QString &name, const QString &message, 
                        QStringList &options, int length)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet(name);
     if (container)
     {
@@ -1159,7 +1202,6 @@ void OSD::NewDialogBox(const QString &name, const QString &message,
     container->SetCache(false);
     container->SetPriority(0);
     container->SetAllowFade(false);
-    container->SetFrameRate(fps);
     AddSet(container, name, false);
 
     OSDTypeText *question = (OSDTypeText *)container->GetType("message");
@@ -1209,12 +1251,14 @@ void OSD::NewDialogBox(const QString &name, const QString &message,
     HighlightDialogSelection(container, availoptions - numoptions);
 
     if (length > 0)
-        container->DisplayFor(length * fps);
+        container->DisplayFor(length * 1000000);
     else
         container->Display();
-    m_setsvisible = true;
 
-    pthread_mutex_unlock(&osdlock);
+    m_setsvisible = true;
+    changed = true;
+
+    osdlock.unlock();
 }
 
 void OSD::HighlightDialogSelection(OSDSet *container, int num)
@@ -1246,18 +1290,21 @@ void OSD::HighlightDialogSelection(OSDSet *container, int num)
 
 void OSD::TurnDialogOff(const QString &name)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
+
     OSDSet *container = GetSet(name);
     if (container)
     {
         container->Hide();
+        changed = true;
     }
-    pthread_mutex_unlock(&osdlock);
+
+    osdlock.unlock();
 }
 
 void OSD::DialogUp(const QString &name)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet(name);
     if (container)
     {
@@ -1271,14 +1318,15 @@ void OSD::DialogUp(const QString &name)
 
             int selected = type->GetPosition() + type->GetOffset();
             HighlightDialogSelection(container, selected);
+            changed = true;
         }
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::DialogDown(const QString &name)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *container = GetSet(name);
     if (container)
     {
@@ -1292,9 +1340,10 @@ void OSD::DialogDown(const QString &name)
 
             int selected = type->GetPosition() + type->GetOffset();
             HighlightDialogSelection(container, selected);
+            changed = true;
         }
     }
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 bool OSD::DialogShowing(const QString &name)
@@ -1334,9 +1383,10 @@ void OSD::ShowEditArrow(long long number, long long totalframes, int type)
     xpos = editarrowRect.left() + xpos;
     int ypos = editarrowRect.top();
 
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
 
-    OSDSet *set = new OSDSet(name, false, vid_width, vid_height, wmult, hmult);
+    OSDSet *set = new OSDSet(name, false, vid_width, vid_height, wmult, hmult,
+                             frameint);
     set->SetAllowFade(false);
     AddSet(set, name, false);
 
@@ -1356,7 +1406,9 @@ void OSD::ShowEditArrow(long long number, long long totalframes, int type)
     set->AddType(image);
     set->Display();
 
-    pthread_mutex_unlock(&osdlock);
+    changed = true;
+
+    osdlock.unlock();
 }
 
 void OSD::HideEditArrow(long long number, int type)
@@ -1364,29 +1416,46 @@ void OSD::HideEditArrow(long long number, int type)
     char name[128];
     sprintf(name, "%lld-%d", number, type);
 
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     OSDSet *set = GetSet(name);
     if (set)
         set->Hide();
-    pthread_mutex_unlock(&osdlock);
+
+    changed = true;
+    osdlock.unlock();
+}
+
+void OSD::HideAll(void)
+{
+    osdlock.lock();
+
+    vector<OSDSet *>::iterator i;
+    for (i = setList->begin(); i != setList->end(); i++)
+        if (*i)
+            (*i)->Hide();
+
+    changed = true;
+    osdlock.unlock();
 }
 
 void OSD::HideSet(const QString &name)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
 
     OSDSet *set = GetSet(name);
     if (set)
     {
         set->Hide();
     }
-    pthread_mutex_unlock(&osdlock);
+
+    changed = true;
+    osdlock.unlock();
 }
 
 void OSD::UpdateEditText(const QString &seek_amount, const QString &deletemarker, 
 		           const QString &edittime, const QString &framecnt)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
 
     QString name = "editmode";
     OSDSet *set = GetSet(name);
@@ -1407,15 +1476,16 @@ void OSD::UpdateEditText(const QString &seek_amount, const QString &deletemarker
 
         set->Display();
         m_setsvisible = true;
+        changed = true;
     }
 
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::DoEditSlider(QMap<long long, int> deleteMap, long long curFrame,
                        long long totalFrames)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
 
     QString name = "editmode";
     OSDSet *set = GetSet(name);
@@ -1477,33 +1547,49 @@ void OSD::DoEditSlider(QMap<long long, int> deleteMap, long long curFrame,
 
         set->Display();
         m_setsvisible = true;
+        changed = true;
     }
 
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 }
 
 void OSD::SetVisible(OSDSet *set, int length)
 {
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     if (length > 0)
-        set->DisplayFor(length * fps);
+        set->DisplayFor(length * 1000000);
     else
         set->Display();
     m_setsvisible = true;
-    pthread_mutex_unlock(&osdlock);
+    changed = true;
+    osdlock.unlock();
 }
 
-void OSD::Display(VideoFrame *frame)
+OSDSurface *OSD::Display(void)
 {
     bool anytodisplay = false;
-    if (!setList || frame->codec != FMT_YV12)
-        return;
+    if (!setList)
+        return NULL;
 
-    unsigned char *yuvptr = frame->buf;
+    bool actuallydraw = false;
+ 
+    if (changed)
+    {
+        actuallydraw = true;
+        changed = false; 
+    }
+
+    drawSurface->SetChanged(false);
+
+    if (actuallydraw)
+    {
+        drawSurface->SetChanged(true);
+        drawSurface->Clear();
+    }
 
     vector<OSDSet *> removeList;
 
-    pthread_mutex_lock(&osdlock);
+    osdlock.lock();
     vector<OSDSet *>::iterator i = setList->begin();
     for (; i != setList->end(); i++)
     {
@@ -1517,12 +1603,19 @@ void OSD::Display(VideoFrame *frame)
                 timedisp->SetText(thetime);
             }
 
-            container->Draw(yuvptr);
+            if (container->Fading())
+                changed = true;
+
+            container->Draw(drawSurface, actuallydraw);
             anytodisplay = true;
-            if (container->GetFramesLeft() == 0 && totalfadeframes > 0)
+            if (container->GetTimeLeft() == 0 && totalfadetime > 0)
             {
-                container->FadeFor(totalfadeframes);
+                container->FadeFor(totalfadetime);
+                changed = true;
             }
+
+            if (container->NeedsUpdate())
+                changed = true;
         }
         else if (container->HasDisplayed())
         {
@@ -1538,9 +1631,14 @@ void OSD::Display(VideoFrame *frame)
         removeList.pop_back();
     }
 
-    pthread_mutex_unlock(&osdlock);
+    osdlock.unlock();
 
     m_setsvisible = anytodisplay;
+
+    if (m_setsvisible)
+        return drawSurface;
+
+    return NULL;
 }
 
 bool OSD::Visible(void)
@@ -1578,7 +1676,7 @@ class comp
 void OSD::AddSet(OSDSet *set, QString name, bool withlock)
 {
     if (withlock)
-        pthread_mutex_lock(&osdlock);
+        osdlock.lock();
 
     setMap[name] = set;
     setList->push_back(set);
@@ -1586,7 +1684,7 @@ void OSD::AddSet(OSDSet *set, QString name, bool withlock)
     sort(setList->begin(), setList->end(), comp());     
 
     if (withlock)
-        pthread_mutex_unlock(&osdlock);
+        osdlock.unlock();
 }
 
 void OSD::RemoveSet(OSDSet *set)
@@ -1602,3 +1700,4 @@ void OSD::RemoveSet(OSDSet *set)
 
     delete set;
 }
+
