@@ -16,6 +16,8 @@ RemoteEncoder::RemoteEncoder(int num, const QString &host, short port)
     remotehost = host;
     remoteport = port;
 
+    lastchannel = "";
+
     controlSock = NULL;
     pthread_mutex_init(&lock, NULL); 
 }
@@ -79,7 +81,7 @@ QSocket *RemoteEncoder::openControlSocket(const QString &host, short port)
 
     if (sock->state() != QSocket::Connected)
     {
-        cout << "Could not connect to server\n";
+        VERBOSE(VB_GENERAL, "Could not connect to server");
         return NULL;
     }
 
@@ -275,6 +277,8 @@ void RemoteEncoder::ToggleInputs(void)
     strlist << "TOGGLE_INPUTS";
 
     SendReceiveStringList(strlist);
+
+    lastchannel = "";
 }
 
 void RemoteEncoder::ToggleChannelFavorite(void)
@@ -292,6 +296,8 @@ void RemoteEncoder::ChangeChannel(int channeldirection)
     strlist << QString::number(channeldirection);
 
     SendReceiveStringList(strlist);
+
+    lastchannel = "";
 }
 
 void RemoteEncoder::SetChannel(QString channel)
@@ -301,6 +307,8 @@ void RemoteEncoder::SetChannel(QString channel)
     strlist << channel;
 
     SendReceiveStringList(strlist);
+
+    lastchannel = "";
 }
 
 int RemoteEncoder::ChangeContrast(bool direction)
@@ -421,6 +429,8 @@ void RemoteEncoder::GetChannelInfo(QString &title, QString &subtitle,
     iconpath = strlist[7];
     channelname = strlist[8];
     chanid = strlist[9];
+
+    lastchannel = channelname;
 }
 
 void RemoteEncoder::GetInputName(QString &inputname)
@@ -433,3 +443,15 @@ void RemoteEncoder::GetInputName(QString &inputname)
     inputname = strlist[0];
 }
 
+QString RemoteEncoder::GetCurrentChannel(void)
+{
+    if (lastchannel == "")
+    {
+        QString dummy;
+        GetChannelInfo(dummy, dummy, dummy, dummy, dummy, dummy,
+                                dummy, dummy, lastchannel, dummy);
+    }
+
+    return lastchannel;
+}
+
