@@ -69,7 +69,7 @@ public:
     virtual void fillSelections(QSqlDatabase* db) {
         clearSelections();
         addSelection(QObject::tr("(unspecified)"), "0");
-        RecordingProfile::fillSelections(db, this);
+        RecordingProfile::fillSelections(db, this, 0);
     };
 };
 
@@ -237,7 +237,7 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
 "channel.channum, channel.callsign, channel.name, "
 "oldrecorded.starttime IS NOT NULL AS oldrecduplicate, program.category, "
 "record.rank, record.recorddups, "
-"recorded.starttime IS NOT NULL as recduplicate "
+"recorded.starttime IS NOT NULL as recduplicate, record.type "
 "FROM record "
 " INNER JOIN channel ON (channel.chanid = program.chanid) "
 " INNER JOIN program ON (program.title = record.title) "
@@ -309,12 +309,20 @@ void ScheduledRecording::findAllProgramsToRecord(QSqlDatabase* db,
              proginfo->chansign = result.value(8).toString();
              proginfo->channame = result.value(9).toString();
 
-             // if recorddups is set then check if recording still exists
-             // in recorded table otherwise check for dup in oldrecorded.
-             if (result.value(13).toInt())
-                 proginfo->duplicate = result.value(14).toInt();
+             if (result.value(15).toInt() == kSingleRecord)
+             {
+                 // force single-recording to be non-duplicate
+                 proginfo->duplicate = false;
+             }
              else
-                 proginfo->duplicate = result.value(10).toInt();
+             {
+                 // if recorddups is set then check if recording still exists
+                 // in recorded table otherwise check for dup in oldrecorded.
+                 if (result.value(13).toInt())
+                     proginfo->duplicate = result.value(14).toInt();
+                 else
+                     proginfo->duplicate = result.value(10).toInt();
+             }
 
              if (proginfo->duplicate)
                  proginfo->recording = false;
@@ -638,8 +646,7 @@ MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent,
                                              const char *name)
 {
     float wmult, hmult;
-    int screenwidth, screenheight;
-    gContext->GetScreenSettings(screenwidth, wmult, screenheight, hmult);
+    gContext->GetScreenSettings(wmult, hmult);
 
     MythDialog* dialog = new ConfigurationDialogWidget(parent, name);
     QVBoxLayout* vbox = new QVBoxLayout(dialog, (int)(20 * wmult));
@@ -657,10 +664,17 @@ MythDialog* ScheduledRecording::dialogWidget(MythMainWindow *parent,
     vbox->addWidget(f);    
 
     vbox->addWidget(type->configWidget(this, dialog));
-    vbox->addWidget(recorddups->configWidget(this, dialog));
-    vbox->addWidget(profile->configWidget(this, dialog));
-    vbox->addWidget(rank->configWidget(this, dialog));
-    vbox->addWidget(autoexpire->configWidget(this, dialog));
+
+    QHBoxLayout* hbox = new QHBoxLayout(vbox, (int)(20 * wmult));
+    QVBoxLayout* vbox1 = new QVBoxLayout(hbox, (int)(20 * wmult));
+    QVBoxLayout* vbox2 = new QVBoxLayout(hbox, (int)(20 * wmult));
+
+    vbox1->addWidget(profile->configWidget(this, dialog));
+    vbox1->addWidget(rank->configWidget(this, dialog));
+
+    vbox2->addWidget(recorddups->configWidget(this, dialog));
+    vbox2->addWidget(autoexpire->configWidget(this, dialog));
+
     vbox->addWidget(maxepisodes->configWidget(this, dialog));
     vbox->addWidget(maxnewest->configWidget(this, dialog));
 
@@ -767,6 +781,6 @@ void ScheduledRecording::setRank(const QString& newrank) {
     rank->setValue(newrank.toInt());
 }
 
-int ScheduledRecording::getProfileID(void) const {
-    return profile->getValue().toInt();
+QString ScheduledRecording::getProfileName(void) const {
+    return profile->getValue();
 }
