@@ -42,12 +42,13 @@ TVRec::TVRec(int capturecardnum)
     rbuffer = NULL;
     nvr = NULL;
     readthreadSock = NULL;
-    readrequest = 0;
     readthreadlive = false;
     m_capturecardnum = capturecardnum;
     ispip = false;
     curRecording = NULL;
     pendingRecording = NULL;
+    profileName = "";
+    autoTranscode = 0;
 
     pthread_mutex_init(&db_lock, NULL);
     pthread_mutex_init(&commLock, NULL);
@@ -184,10 +185,10 @@ int TVRec::StartRecording(ProgramInfo *rcinfo)
 
     if (changeState)
     {
-        VERBOSE("backend still changing state, waiting..");
+        VERBOSE(VB_RECORD, "backend still changing state, waiting..");
         while (changeState)
             usleep(50);
-        VERBOSE("changing state finished, starting now");
+        VERBOSE(VB_RECORD, "changing state finished, starting now");
     }
 
     if (internalState == kState_WatchingLiveTV && !cancelNextRecording)
@@ -426,7 +427,9 @@ void TVRec::HandleStateChange(void)
     }
     else
     {
-        printf("Changing from %s to %s\n", origname.ascii(), statename.ascii());
+        QString trans = QString("Changing from %1 to %2")
+                               .arg(origname).arg(statename);
+        VERBOSE(VB_GENERAL, trans);
     }
  
     if (startRecorder)
@@ -439,25 +442,34 @@ void TVRec::HandleStateChange(void)
 
         prematurelystopped = false;
 
-        if (!profile.loadByName(db_conn, gContext->GetHostName()))
+        if (curRecording)
         {
-            if (curRecording) 
+            profileName = curRecording->GetScheduledRecording(
+                                                db_conn)->getProfileName();
+            bool foundProf = false;
+            if (profileName != NULL)
+                foundProf = profile.loadByCard(db_conn, profileName,
+                                               m_capturecardnum);
+
+            if (!foundProf)
             {
-                int profileID = curRecording->GetScheduledRecording(db_conn)
-                                            ->getProfileID();
-                if (profileID > 0)
-                    profile.loadByID(db_conn, profileID);
-                else
-                    profile.loadByName(db_conn, "Default");
-            } 
-            else 
-            {
-                profile.loadByName(db_conn, "Live TV");
+                profileName = QString("Default");
+                profile.loadByCard(db_conn, profileName, m_capturecardnum);
             }
         }
+        else
+        {
+            profileName = "Live TV";
+            profile.loadByCard(db_conn, profileName, m_capturecardnum);
+        }
+        QString msg = QString("Using profile '%1' to record").arg(profileName);
+        VERBOSE(VB_RECORD, msg);
 
         pthread_mutex_unlock(&db_lock);
 
+        // Determine whether to automatically run the transcoder or not
+        autoTranscode = profile.byName("autotranscode")->getValue().toInt();
+
         SetupRecorder(profile);
         nvr->SetRecording(curRecording);
         nvr->SetDB(db_conn, &db_lock);
@@ -500,6 +512,20 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
         nvr->SetOption("tvformat", gContext->GetSetting("TVFormat"));
         nvr->SetOption("vbiformat", gContext->GetSetting("VbiFormat"));
 
+        SetOption(profile, "mpeg2bitrate");
+        SetOption(profile, "mpeg2maxbitrate");
+        nvr->SetOption("mpeg2streamtype", 
+                       profile.byName("mpeg2streamtype")->getValue());
+        nvr->SetOption("mpeg2aspectratio", 
+                       profile.byName("mpeg2aspectratio")->getValue());
+
+        SetOption(profile, "samplerate");
+        nvr->SetOption("mpeg2audtype", 
+                       profile.byName("mpeg2audtype")->getValue());
+        SetOption(profile, "mpeg2audbitratel1");
+        SetOption(profile, "mpeg2audbitratel2");
+        SetOption(profile, "mpeg2audvolume");
+
         SetOption(profile, "width");
         SetOption(profile, "height");
 
@@ -532,7 +558,7 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
         return;
     }
 
-    // V4L from here on
+    // V4L/MJPEG from here on
 
     nvr = new NuppelVideoRecorder(channel);
 
@@ -572,7 +598,7 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
         SetOption(profile, "hardwaremjpegquality");
         SetOption(profile, "hardwaremjpeghdecimation");
         SetOption(profile, "hardwaremjpegvdecimation");
-    } 
+    }
     else 
         cerr << "Unknown video codec: " << setting << endl;
 
@@ -616,6 +642,7 @@ RecorderBase* TVRec::GetRecorder()
 void TVRec::TeardownRecorder(bool killFile)
 {
     QMap<long long, int> blank_frame_map;
+    QString oldProfileName = profileName;
 
     int filelen = -1;
 
@@ -636,6 +663,7 @@ void TVRec::TeardownRecorder(bool killFile)
         gContext->dispatch(me);
 
         nvr->StopRecording();
+        profileName = "";
 
         if (prevRecording && !killFile)
             nvr->GetBlankFrameMap(blank_frame_map);
@@ -647,11 +675,12 @@ void TVRec::TeardownRecorder(bool killFile)
 
     if (rbuffer)
     {
-        if (readthreadlive)
-            KillReadThread();
-
+        rbuffer->StopReads();
+        readthreadLock.lock();
         delete rbuffer;
+        readthreadlive = false;
         rbuffer = NULL;
+        readthreadLock.unlock();
     }
 
     if (killFile)
@@ -685,11 +714,12 @@ void TVRec::TeardownRecorder(bool killFile)
             if (gContext->GetNumSetting("AutoCommercialFlag", 1))
                 FlagCommercials();
 
-            if (gContext->GetNumSetting("TranscoderAutoRun", 0))
+            if (autoTranscode)
             {
-                QString message = QString("LOCAL_TRANSCODE %1 %2")
+                QString message = QString("LOCAL_TRANSCODE %1 %2 %3")
                            .arg(prevRecording->chanid)
-                           .arg(prevRecording->startts.toString(Qt::ISODate));
+                           .arg(prevRecording->startts.toString(Qt::ISODate))
+                           .arg(oldProfileName);
                 MythEvent me(message);
                 gContext->dispatch(me);
             }
@@ -1609,8 +1639,7 @@ void TVRec::PauseRecorder(void)
 
 void TVRec::ToggleInputs(void)
 {
-    while (!nvr->GetPause())
-        usleep(5);
+    nvr->WaitForPause();
 
     PauseClearRingBuffer();
 
@@ -1626,13 +1655,12 @@ void TVRec::ToggleInputs(void)
 
 void TVRec::ChangeChannel(int channeldirection)
 {
-    while (!nvr->GetPause())
-        usleep(5);
+    nvr->WaitForPause();
 
     PauseClearRingBuffer();
 
     rbuffer->Reset();
-    
+
     if (channeldirection == CHANNEL_DIRECTION_FAVORITE)
         channel->NextFavorite();
     else if (channeldirection == CHANNEL_DIRECTION_UP)
@@ -1707,7 +1735,7 @@ void TVRec::ToggleChannelFavorite(void)
                            .arg(favid);
 
         query = db_conn->exec(thequery);
-        cout << "Removing Favorite.\n";
+        VERBOSE(VB_RECORD, "Removing Favorite.");
     }
     else
     {
@@ -1716,7 +1744,7 @@ void TVRec::ToggleChannelFavorite(void)
                            .arg(chanid);
 
         query = db_conn->exec(thequery);
-        cout << "Adding Favorite.\n";
+        VERBOSE(VB_RECORD, "Adding Favorite.");
     }
     pthread_mutex_unlock(&db_lock);
 }
@@ -1747,8 +1775,7 @@ int TVRec::ChangeHue(bool direction)
 
 void TVRec::SetChannel(QString name)
 {
-    while (!nvr->GetPause())
-        usleep(5);
+    nvr->WaitForPause();
 
     PauseClearRingBuffer();
 
@@ -1885,7 +1912,7 @@ void TVRec::UnpauseRingBuffer(void)
         return;
 
     rbuffer->StartReads();
-    pthread_mutex_unlock(&readthreadLock);
+    readthreadLock.unlock();
 }
 
 void TVRec::PauseClearRingBuffer(void)
@@ -1894,7 +1921,7 @@ void TVRec::PauseClearRingBuffer(void)
         return;
 
     rbuffer->StopReads();
-    pthread_mutex_lock(&readthreadLock);
+    readthreadLock.lock();
 }
 
 long long TVRec::SeekRingBuffer(long long curpos, long long pos, int whence)
@@ -1919,117 +1946,53 @@ long long TVRec::SeekRingBuffer(long long curpos, long long pos, int whence)
     return ret;
 }
 
-void TVRec::SpawnReadThread(QSocket *sock)
-{
-    if (readthreadlive)
-        return;
-
-    pthread_mutex_init(&readthreadLock, NULL);
-    readthreadSock = sock;
-    readthreadlive = false;
-
-    pthread_create(&readthread, NULL, ReadThread, this);
-
-    while (!readthreadlive)
-        usleep(50);
-}
-
-void TVRec::KillReadThread(void)
-{
-    if (readthreadlive)
-    {
-        readthreadlive = false;
-        rbuffer->StopReads();
-        pthread_join(readthread, NULL);
-        readthreadSock = NULL;
-    }
-}
-
 QSocket *TVRec::GetReadThreadSocket(void)
 {
     return readthreadSock;
 }
 
+void TVRec::SetReadThreadSock(QSocket *sock)
+{
+    if ((readthreadlive && sock) || (!readthreadlive && !sock))
+        return;
+
+    if (sock)
+    {
+        readthreadSock = sock;
+        readthreadlive = true;
+    }
+    else
+    {
+        readthreadlive = false;
+        if (rbuffer)
+            rbuffer->StopReads();
+        readthreadLock.lock();
+        readthreadLock.unlock();
+    }
+}
+
 void TVRec::RequestRingBufferBlock(int size)
 {
     if (!readthreadlive)
         return;
 
+    char buffer[256001];
+
     if (size > 256000)
         size = 256000;
 
-    bool locked = false;
-    QDateTime curtime = QDateTime::currentDateTime();
-    curtime = curtime.addSecs(15);
+    readthreadLock.lock();
 
-    while (QDateTime::currentDateTime() < curtime)
+    while (size > 0 && !rbuffer->GetStopReads() && readthreadlive)
     {
-        locked = pthread_mutex_trylock(&readthreadLock);
-        if (locked)
-            break;
-        usleep(50);
-    }
-
-    if (!locked)
-    {
-        cerr << "Backend stopped in RequestRingBufferBlock\n";
-        rbuffer->StopReads();
-        return;
-    }
-
-    readrequest = size; 
-    pthread_mutex_unlock(&readthreadLock);
-
-    curtime = QDateTime::currentDateTime();
-    curtime = curtime.addSecs(15);
-
-    while (readrequest > 0 && readthreadlive)
-    {
-        usleep(500);
-
-        if (QDateTime::currentDateTime() > curtime)
-        {
-            cerr << "Backend stuffed up in RequestRingBufferBlock\n";
-            rbuffer->StopReads();
-            break;
-        }
-    }
-}
-
-void TVRec::DoReadThread(void)
-{
-    readthreadlive = true;
-
-    char *buffer = new char[256001];
-
-    while (readthreadlive && rbuffer)
-    {
-        while (readrequest == 0 && readthreadlive)
-            usleep(5000);
-
-        if (!readthreadlive)
-            break;
-
-        pthread_mutex_lock(&readthreadLock);
-
-        int ret = rbuffer->Read(buffer, readrequest);
+        int ret = rbuffer->Read(buffer, size);
         if (!rbuffer->GetStopReads() && ret > 0)
             WriteBlock(readthreadSock, buffer, ret);
 
-        readrequest -= ret;
-
-        pthread_mutex_unlock(&readthreadLock);
+        size -= ret;
     }
 
-    delete [] buffer;
-}
-
-void *TVRec::ReadThread(void *param)
-{
-    TVRec *thetv = (TVRec *)param;
-    thetv->DoReadThread();
-
-    return NULL;
+    readthreadLock.unlock();
 }
 
 void TVRec::DoFlagCommercialsThread(void)
