@@ -17,7 +17,9 @@
 using namespace std;
 
 #include "videoout_xv.h"
+#include "osdsurface.h"
 #include "../libmyth/util.h"
+#include "mythcontext.h"
 
 extern "C" {
 #include "../libavcodec/avcodec.h"
@@ -41,6 +43,11 @@ extern int      XShmGetEventBase(Display*);
 extern XvImage  *XvShmCreateImage(Display*, XvPortID, int, char*, int, int, XShmSegmentInfo*);
 }
 
+const int kNumBuffers = 31;
+const int kNeedFreeFrames = 1;
+const int kPrebufferFrames = 12;
+const int kKeepPrebuffer = 2;
+
 struct XvData
 {
     Window XJ_root;
@@ -59,7 +66,7 @@ struct XvData
 #define GUID_I420_PLANAR 0x30323449
 #define GUID_YV12_PLANAR 0x32315659
 
-int XJ_error_catcher(Display * d, XErrorEvent * xeev)
+static int XJ_error_catcher(Display * d, XErrorEvent * xeev)
 {
   d = d; 
   xeev = xeev;
@@ -67,20 +74,36 @@ int XJ_error_catcher(Display * d, XErrorEvent * xeev)
 }
 
 VideoOutputXv::VideoOutputXv(void)
+             : VideoOutput()
 {
     XJ_started = 0; 
     xv_port = -1; 
     scratchspace = NULL; 
 
+    pauseFrame.buf = NULL;
+
     data = new XvData();
 }
 
 VideoOutputXv::~VideoOutputXv()
 {
+    if (pauseFrame.buf)
+        delete [] pauseFrame.buf;
+
     Exit();
     delete data;
 }
 
+void VideoOutputXv::AspectChanged(float aspect)
+{
+    pthread_mutex_lock(&lock);
+
+    VideoOutput::AspectChanged(aspect);
+    MoveResize();
+
+    pthread_mutex_unlock(&lock);
+}
+
 void VideoOutputXv::InputChanged(int width, int height, float aspect)
 {
     pthread_mutex_lock(&lock);
@@ -101,10 +124,22 @@ void VideoOutputXv::InputChanged(int width, int height, float aspect)
     else
     {
         DeleteXBuffers();
-        CreateShmBuffers();
+        CreateXBuffers();
     }
 
     MoveResize();
+
+    scratchFrame = &(vbuffers[kNumBuffers]);
+
+    if (pauseFrame.buf)
+        delete [] pauseFrame.buf;
+
+    pauseFrame.height = scratchFrame->height;
+    pauseFrame.width = scratchFrame->width;
+    pauseFrame.bpp = scratchFrame->bpp;
+    pauseFrame.size = scratchFrame->size;
+    pauseFrame.buf = new unsigned char[pauseFrame.size];
+
     pthread_mutex_unlock(&lock);
 }
 
@@ -128,10 +163,9 @@ int VideoOutputXv::GetRefreshRate(void)
     return (int)rate;
 }
 
-bool VideoOutputXv::Init(int width, int height, float aspect, int num_buffers, 
-                         VideoFrame *out_buffers, unsigned int winid,
-                         int winx, int winy, int winw, int winh, 
-                         unsigned int embedid)
+bool VideoOutputXv::Init(int width, int height, float aspect, 
+                         unsigned int winid, int winx, int winy, int winw, 
+                         int winh, unsigned int embedid)
 {
     pthread_mutex_init(&lock, NULL);
 
@@ -145,8 +179,10 @@ bool VideoOutputXv::Init(int width, int height, float aspect, int num_buffers,
 
     XvAdaptorInfo *ai;
 
-    VideoOutput::Init(width, height, aspect, num_buffers, out_buffers, winid,
-                      winx, winy, winw, winh, embedid);
+    VideoOutput::InitBuffers(kNumBuffers, true, kNeedFreeFrames,
+                             kPrebufferFrames, kKeepPrebuffer);
+    VideoOutput::Init(width, height, aspect, winid, winx, winy, winw, winh, 
+                      embedid);
 
     data->XJ_disp = XOpenDisplay(NULL);
     if (!data->XJ_disp) 
@@ -181,7 +217,7 @@ bool VideoOutputXv::Init(int width, int height, float aspect, int num_buffers,
         ai = NULL;
     }
     else
-        if ( ai )
+        if (ai)
         {
             for (i = 0; i < p_num_adaptors; i++) 
             {
@@ -229,9 +265,6 @@ bool VideoOutputXv::Init(int width, int height, float aspect, int num_buffers,
 
     data->XJ_curwin = data->XJ_win = winid;
     
-    if (embedid > 0)
-        data->XJ_curwin = data->XJ_win = embedid;
-
     old_handler = XSetErrorHandler(XJ_error_catcher);
     XSync(data->XJ_disp, 0);
 
@@ -319,8 +352,24 @@ bool VideoOutputXv::Init(int width, int height, float aspect, int num_buffers,
         return false;
     }
 
+    scratchFrame = &(vbuffers[kNumBuffers]);
+
+    pauseFrame.height = scratchFrame->height;
+    pauseFrame.width = scratchFrame->width;
+    pauseFrame.bpp = scratchFrame->bpp;
+    pauseFrame.size = scratchFrame->size;
+    pauseFrame.buf = new unsigned char[pauseFrame.size];
+
     MoveResize();
-  
+ 
+    if (gContext->GetNumSetting("UseOutputPictureControls", 0))
+    {
+        ChangePictureAttribute(kPictureAttribute_Brightness, brightness);
+        ChangePictureAttribute(kPictureAttribute_Contrast, contrast);
+        ChangePictureAttribute(kPictureAttribute_Colour, colour);
+        ChangePictureAttribute(kPictureAttribute_Hue, hue);
+    }
+
     XJ_started = true;
 
     return true;
@@ -328,8 +377,8 @@ bool VideoOutputXv::Init(int width, int height, float aspect, int num_buffers,
 
 bool VideoOutputXv::CreateXvBuffers(void)
 {
-    data->XJ_SHMInfo = new XShmSegmentInfo[numbuffers];
-    for (int i = 0; i < numbuffers; i++)
+    data->XJ_SHMInfo = new XShmSegmentInfo[numbuffers + 1];
+    for (int i = 0; i < numbuffers + 1; i++)
     {
         XvImage *image = XvShmCreateImage(data->XJ_disp, xv_port,
                                           colorid, 0, XJ_width, XJ_height,
@@ -356,12 +405,12 @@ bool VideoOutputXv::CreateXvBuffers(void)
 
         XShmAttach(data->XJ_disp, &(data->XJ_SHMInfo)[i]);
 
-        videoframes[i].buf = (unsigned char *)image->data;
-        videoframes[i].height = XJ_height;
-        videoframes[i].width = XJ_width;
-        videoframes[i].bpp = 12;
-        videoframes[i].size = XJ_height * XJ_width * 3 / 2;
-        videoframes[i].codec = FMT_YV12;
+        vbuffers[i].buf = (unsigned char *)image->data;
+        vbuffers[i].height = XJ_height;
+        vbuffers[i].width = XJ_width;
+        vbuffers[i].bpp = 12;
+        vbuffers[i].size = XJ_height * XJ_width * 3 / 2;
+        vbuffers[i].codec = FMT_YV12;
     }
 
     XSync(data->XJ_disp, 0);
@@ -409,14 +458,14 @@ bool VideoOutputXv::CreateShmBuffers(void)
         return false;
     }
 
-    for (int i = 0; i < numbuffers; i++)
+    for (int i = 0; i < numbuffers + 1; i++)
     {
-        videoframes[i].height = XJ_height;
-        videoframes[i].width = XJ_width;
-        videoframes[i].bpp = 12;
-        videoframes[i].size = XJ_height * XJ_width * 3 / 2;
-        videoframes[i].codec = FMT_YV12;
-        videoframes[i].buf = new unsigned char[videoframes[i].size];
+        vbuffers[i].height = XJ_height;
+        vbuffers[i].width = XJ_width;
+        vbuffers[i].bpp = 12;
+        vbuffers[i].size = XJ_height * XJ_width * 3 / 2;
+        vbuffers[i].codec = FMT_YV12;
+        vbuffers[i].buf = new unsigned char[vbuffers[i].size];
     }
 
     XSync(data->XJ_disp, 0);
@@ -435,14 +484,14 @@ bool VideoOutputXv::CreateXBuffers(void)
 
     data->fallbackImage = image;
 
-    for (int i = 0; i < numbuffers; i++)
+    for (int i = 0; i < numbuffers + 1; i++)
     {
-        videoframes[i].height = XJ_height;
-        videoframes[i].width = XJ_width;
-        videoframes[i].bpp = 12;
-        videoframes[i].size = XJ_height * XJ_width * 3 / 2;
-        videoframes[i].codec = FMT_YV12;
-        videoframes[i].buf = new unsigned char[videoframes[i].size];
+        vbuffers[i].height = XJ_height;
+        vbuffers[i].width = XJ_width;
+        vbuffers[i].bpp = 12;
+        vbuffers[i].size = XJ_height * XJ_width * 3 / 2;
+        vbuffers[i].codec = FMT_YV12;
+        vbuffers[i].buf = new unsigned char[vbuffers[i].size];
     }
 
     XSync(data->XJ_disp, 0);
@@ -504,10 +553,10 @@ void VideoOutputXv::DeleteShmBuffers()
 
     delete [] (data->XJ_SHMInfo);
 
-    for (int i = 0; i < numbuffers; i++)
+    for (int i = 0; i < numbuffers + 1; i++)
     {
-        delete [] videoframes[i].buf;
-        videoframes[i].buf = NULL;
+        delete [] vbuffers[i].buf;
+        vbuffers[i].buf = NULL;
     }
 }
 
@@ -515,10 +564,10 @@ void VideoOutputXv::DeleteXBuffers()
 {
     XFree(data->fallbackImage);
 
-    for (int i = 0; i < numbuffers; i++)
+    for (int i = 0; i < numbuffers + 1; i++)
     {
-        delete [] videoframes[i].buf;
-        videoframes[i].buf = NULL;
+        delete [] vbuffers[i].buf;
+        vbuffers[i].buf = NULL;
     }
 }
 
@@ -550,6 +599,9 @@ void VideoOutputXv::StopEmbedding(void)
 
 void VideoOutputXv::PrepareFrame(VideoFrame *buffer)
 {
+    if (!buffer)
+        buffer = scratchFrame;
+
     if (xv_port != -1)
     {
         pthread_mutex_lock(&lock);
@@ -671,5 +723,112 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer)
 
 void VideoOutputXv::Show()
 {
+    pthread_mutex_lock(&lock);
     XSync(data->XJ_disp, False);
+    pthread_mutex_unlock(&lock);
+}
+
+void VideoOutputXv::DrawUnusedRects(void)
+{
+    XSetForeground(data->XJ_disp, data->XJ_gc, XJ_black);
+  
+    XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, dispx, dispy, 
+                   dispw, dispyoff);
+    XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, dispx, 
+                   dispyoff + disphoff, dispw, dispyoff);
+}
+
+void VideoOutputXv::UpdatePauseFrame(void)
+{
+    pthread_mutex_lock(&lock);
+
+    VideoFrame *pauseb = scratchFrame;
+    if (usedVideoBuffers.count() > 0)
+        pauseb = usedVideoBuffers.head();
+    memcpy(pauseFrame.buf, pauseb->buf, pauseb->size);
+
+    pthread_mutex_unlock(&lock);
+}
+
+void VideoOutputXv::ProcessFrame(VideoFrame *frame, OSD *osd,
+                                 vector<VideoFilter *> &filterList,
+                                 NuppelVideoPlayer *pipPlayer)
+{
+    pthread_mutex_lock(&lock);
+
+    if (!frame)
+    {
+        frame = scratchFrame;
+        memcpy(scratchFrame->buf, pauseFrame.buf, pauseFrame.size);
+    }
+
+    process_video_filters(frame, &filterList[0], filterList.size());
+
+    ShowPip(frame, pipPlayer);
+    DisplayOSD(frame, osd);
+
+    pthread_mutex_unlock(&lock);
+}
+
+int VideoOutputXv::ChangePictureAttribute(int attributeType, int newValue)
+{
+    int value;
+    int i, howmany, port_min, port_max, range;
+    char *attrName = NULL;
+    Atom attribute;
+    XvAttribute *attributes;
+
+    switch (attributeType) {
+        case kPictureAttribute_Brightness:
+            attrName = "XV_BRIGHTNESS";
+            break;
+        case kPictureAttribute_Contrast:
+            attrName = "XV_CONTRAST";
+            break;
+        case kPictureAttribute_Colour:
+            attrName = "XV_SATURATION";
+            break;
+        case kPictureAttribute_Hue:
+            attrName = "XV_HUE";
+            break;
+    }
+
+    if (!attrName)
+        return -1;
+
+    if (newValue < 0) newValue = 0;
+    if (newValue >= 100) newValue = 99;
+
+    pthread_mutex_lock(&lock);
+
+    attribute = XInternAtom (data->XJ_disp, attrName, False);
+    if (!attribute) {
+        pthread_mutex_unlock(&lock);
+        return -1;
+    }
+    attributes = XvQueryPortAttributes(data->XJ_disp, xv_port, &howmany);
+    if (!attributes) {
+        pthread_mutex_unlock(&lock);
+        return -1;
+    }
+
+    for (i = 0; i < howmany; i++) {
+        if(!strcmp(attrName, attributes[i].name)) {
+            port_min = attributes[i].min_value;
+            port_max = attributes[i].max_value;
+            range = port_max - port_min;
+
+            value = (int) (port_min + (range/100.0) * newValue);
+
+            XvSetPortAttribute(data->XJ_disp, xv_port, attribute, value);
+
+            pthread_mutex_unlock(&lock);
+
+            return newValue;
+        }
+    }
+
+    pthread_mutex_unlock(&lock);
+    
+    return -1;
 }
