@@ -5,25 +5,36 @@ struct XvData;
 
 #include "videooutbase.h"
 
+class NuppelVideoPlayer;
+
 class VideoOutputXv : public VideoOutput
 {
   public:
     VideoOutputXv();
    ~VideoOutputXv();
 
-    bool Init(int width, int height, float aspect, int num_buffers, 
-              VideoFrame *out_buffers, unsigned int winid,
+    bool Init(int width, int height, float aspect, unsigned int winid,
               int winx, int winy, int winw, int winh, unsigned int embedid = 0);
     void PrepareFrame(VideoFrame *buffer);
     void Show(void);
 
     void InputChanged(int width, int height, float aspect);
+    void AspectChanged(float aspect);
 
     void EmbedInWidget(unsigned long wid, int x, int y, int w, int h);
     void StopEmbedding(void);
 
     int GetRefreshRate(void);
 
+    void DrawUnusedRects(void);
+
+    void UpdatePauseFrame(void);
+    void ProcessFrame(VideoFrame *frame, OSD *osd,
+                      vector<VideoFilter *> &filterList,
+                      NuppelVideoPlayer *pipPlayer);
+
+    int ChangePictureAttribute(int attributeType, int newValue);
+
   private:
     void Exit(void);
     bool CreateXvBuffers(void);
@@ -51,6 +62,9 @@ class VideoOutputXv : public VideoOutput
     unsigned char *scratchspace;
 
     pthread_mutex_t lock;
+
+    VideoFrame *scratchFrame;
+    VideoFrame pauseFrame;
 };
 
 #endif
