@@ -11,13 +11,13 @@ class VideoOutputXvMC : public VideoOutput
     VideoOutputXvMC();
    ~VideoOutputXvMC();
 
-    bool Init(int width, int height, float aspect, int num_buffers, 
-              VideoFrame *out_buffers, unsigned int winid,
+    bool Init(int width, int height, float aspect, unsigned int winid,
               int winx, int winy, int winw, int winh, unsigned int embedid = 0);
     void PrepareFrame(VideoFrame *buffer);
     void Show(void);
 
     void InputChanged(int width, int height, float aspect);
+    void AspectChanged(float aspect);
 
     void EmbedInWidget(unsigned long wid, int x, int y, int w, int h);
     void StopEmbedding(void);
@@ -26,6 +26,15 @@ class VideoOutputXvMC : public VideoOutput
 
     void DrawSlice(VideoFrame *frame, int x, int y, int w, int h);
 
+    void DrawUnusedRects(void);
+
+    void UpdatePauseFrame(void);
+    void ProcessFrame(VideoFrame *frame, OSD *osd,
+                      vector<VideoFilter *> &filterList,
+                      NuppelVideoPlayer *pipPlayer);
+
+    int ChangePictureAttribute(int attributeType, int newValue);
+
   private:
     void Exit(void);
     bool CreateXvMCBuffers(void);
@@ -46,6 +55,8 @@ class VideoOutputXvMC : public VideoOutput
     int colorid;
 
     pthread_mutex_t lock;
+
+    int colorkey;
 };
 
 #endif
