@@ -172,12 +172,15 @@ public:
         addTarget("tv_grab_sn", new XMLTV_generic_config(parent, "tv_grab_sn"));
         grabber->addSelection("Sweden/Norway","tv_grab_sn");
 
-        addTarget("tv_grab_uk", new XMLTV_uk_config(parent));
+        addTarget("tv_grab_uk", new XMLTV_generic_config(parent, "tv_grab_uk"));
         grabber->addSelection("United Kingdom","tv_grab_uk");
 
         addTarget("tv_grab_uk_rt", new XMLTV_generic_config(parent, "tv_grab_uk_rt"));
         grabber->addSelection("United Kingdom (alternative)","tv_grab_uk_rt");
 
+        addTarget("tv_grab_au", new XMLTV_generic_config(parent, "tv_grab_au"));
+        grabber->addSelection("Australia", "tv_grab_au");
+
         addTarget("tv_grab_nz", new XMLTV_generic_config(parent, "tv_grab_nz"));
         grabber->addSelection("New Zealand", "tv_grab_nz");
 
@@ -277,6 +280,8 @@ public:
         fillSelectionsFromDir(dev);
         dev.setPath("/dev/v4l");
         fillSelectionsFromDir(dev);
+        dev.setPath("/dev/dtv");
+        fillSelectionsFromDir(dev);
     };
 
     static QStringList probeInputs(QString device);
@@ -333,35 +338,178 @@ public slots:
     void fillSelections(const QString& device);
 };
 
+class DVBCardNum: public SpinBoxSetting, public CCSetting {
+public:
+    DVBCardNum(const CaptureCard& parent):
+        SpinBoxSetting(0,3,1),
+        CCSetting(parent, "videodevice") {
+        setLabel("DVB Card Number");
+        setHelpText("When you change this setting, the text below should change"
+                    " to the name and type of your card. If the card cannot be"
+                    " opened, an error message will be displayed.");
+    };
+};
+
+class DVBCardType: public LabelSetting, public TransientStorage {
+public:
+    DVBCardType() {
+        setLabel("Card Type");
+    };
+};
+
+class DVBCardName: public LabelSetting, public TransientStorage {
+public:
+    DVBCardName() {
+        setLabel("Card Name");
+    };
+};
+
+class DVBUseTS: public CheckBoxSetting, public CCSetting {
+public:
+    DVBUseTS(const CaptureCard& parent):
+        CCSetting(parent, "use_ts") {
+        setLabel("Use Transport Stream for recording");
+        setHelpText("This option is to get around filtering limitations on some"
+                    " DVB cards. Most users should not activate this option,"
+                    " because of compatability problems.");
+    };
+};
+
+class DVBAudioDevice: public LineEditSetting, public CCSetting {    
+    Q_OBJECT
+public:
+    DVBAudioDevice(const CaptureCard& parent):
+        CCSetting(parent,"audiodevice") {
+        setVisible(false);
+    };
+
+    void save(QSqlDatabase* db) {
+        changed = true;
+        settingValue = "";
+        SimpleDBStorage::save(db);
+    };
+};
+
+class DVBVbiDevice: public LineEditSetting, public CCSetting {
+    Q_OBJECT
+public:
+    DVBVbiDevice(const CaptureCard& parent):
+        CCSetting(parent,"vbidevice") {
+        setVisible(false);
+    };
+    void save(QSqlDatabase* db) {
+        changed = true;
+        settingValue = "";
+        SimpleDBStorage::save(db);
+    };
+};
+
+class DVBDefaultInput: public LineEditSetting, public CCSetting {
+    Q_OBJECT
+public:
+    DVBDefaultInput(const CaptureCard& parent):
+        CCSetting(parent,"defaultinput") {
+        setVisible(false);
+    };
+    void save(QSqlDatabase* db) {
+        changed = true;
+        settingValue = "DVBInput";
+        SimpleDBStorage::save(db);
+    };
+};
+
 class CardType: public ComboBoxSetting, public CCSetting {
 public:
     CardType(const CaptureCard& parent);
+    static void fillSelections(SelectSetting* setting);
+};
+
+class V4LConfigurationGroup: public VerticalConfigurationGroup {
+public:
+    V4LConfigurationGroup(CaptureCard& a_parent):
+        VerticalConfigurationGroup(false),
+        parent(a_parent) {
+        setUseLabel(false);
+
+        VideoDevice* device;
+        TunerCardInput* input;
+
+        addChild(device = new VideoDevice(parent));
+        addChild(new VbiDevice(parent));
+        addChild(new AudioDevice(parent));
+        addChild(new AudioRateLimit(parent));
+        addChild(input = new TunerCardInput(parent));
+
+        connect(device, SIGNAL(valueChanged(const QString&)),
+                input, SLOT(fillSelections(const QString&)));
+        input->fillSelections(device->getValue());
+    };
+private:
+    CaptureCard& parent;
+};
+
+class DVBConfigurationGroup: public VerticalConfigurationGroup {
+    Q_OBJECT
+public:
+    DVBConfigurationGroup(CaptureCard& a_parent):
+        VerticalConfigurationGroup(false),
+        parent(a_parent) {
+        setUseLabel(false);
+
+        DVBCardNum* cardnum = new DVBCardNum(parent);
+        cardname = new DVBCardName();
+        cardtype = new DVBCardType();
+
+        addChild(cardnum);
+        addChild(cardname);
+        addChild(cardtype);
+        addChild(new DVBUseTS(parent));
+        addChild(new DVBAudioDevice(parent));
+        addChild(new DVBVbiDevice(parent));
+        addChild(new DVBDefaultInput(parent));
+
+        connect(cardnum, SIGNAL(valueChanged(const QString&)),
+                this, SLOT(probeCard(const QString&)));
+        cardnum->setValue(0);
+    };
+
+public slots:
+    void probeCard(const QString& cardNumber);
+
+private:
+    CaptureCard& parent;
+
+    DVBCardName* cardname;
+    DVBCardType* cardtype;
+};
+
+class CaptureCardGroup: public VerticalConfigurationGroup,
+                        public TriggeredConfigurationGroup {
+public:
+    CaptureCardGroup(CaptureCard& parent) {
+        CardType* cardtype = new CardType(parent);
+        addChild(cardtype);
+        setTrigger(cardtype);
+        setSaveAll(false);
+
+        addTarget("V4L", new V4LConfigurationGroup(parent));
+        addTarget("MPEG", new V4LConfigurationGroup(parent));
+        addTarget("MJPEG", new V4LConfigurationGroup(parent));
+        addTarget("HDTV", new V4LConfigurationGroup(parent));
+        addTarget("DVB", new DVBConfigurationGroup(parent));
+    };
 };
 
 class CaptureCard: public ConfigurationWizard {
 public:
     CaptureCard() {
-        VideoDevice* device;
-        TunerCardInput* input;
-
         // must be first
         addChild(id = new ID());
 
-        ConfigurationGroup *devices = new VerticalConfigurationGroup(false);
-        devices->setLabel("Capture card");
-        devices->addChild(device = new VideoDevice(*this));
-        devices->addChild(new VbiDevice(*this));
-        devices->addChild(new AudioDevice(*this));
-        devices->addChild(new AudioRateLimit(*this));
-        devices->addChild(input = new TunerCardInput(*this));
-        devices->addChild(new CardType(*this));
-        addChild(devices);
+        CaptureCardGroup *cardgroup = new CaptureCardGroup(*this);
+        addChild(cardgroup);
 
         addChild(new Hostname(*this));
-
-        connect(device, SIGNAL(valueChanged(const QString&)),
-                input, SLOT(fillSelections(const QString&)));
-        input->fillSelections(device->getValue());
     };
 
     int getCardID(void) const {
