@@ -2,6 +2,7 @@
 #include <qsqldatabase.h>
 #include <qdatetime.h>
 #include <qfile.h>
+#include <qdir.h>
 #include <qurl.h>
 #include <qthread.h>
 #include <qwaitcondition.h>
@@ -169,7 +170,9 @@ void MainServer::ProcessRequest(QSocket *sock)
         return;
 
     QStringList listline;
-    ReadStringList(sock, listline);
+    if (!ReadStringList(sock, listline))
+        return;
+
     QString line = listline[0];
 
     line = line.simplifyWhiteSpace();
@@ -227,6 +230,10 @@ void MainServer::ProcessRequest(QSocket *sock)
     {
         HandleQueueTranscode(listline, pbs, TRANSCODE_STOP);
     }
+    else if (command == "STOP_RECORDING")
+    {
+        HandleStopRecording(listline, pbs);
+    }
     else if (command == "DELETE_RECORDING")
     {
         HandleDeleteRecording(listline, pbs);
@@ -287,6 +294,10 @@ void MainServer::ProcessRequest(QSocket *sock)
     {
         HandleGenPreviewPixmap(listline, pbs);
     }
+    else if (command == "QUERY_ISRECORDING") 
+    {
+        HandleIsRecording(listline, pbs);
+    }
     else if (command == "MESSAGE")
     {
         HandleMessage(listline, pbs);
@@ -391,7 +402,7 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
     if (commands[1] == "Playback")
     {
         bool wantevents = commands[3].toInt();
-        VERBOSE("MainServer::HandleAnnounce Playback");
+        VERBOSE(VB_GENERAL, "MainServer::HandleAnnounce Playback");
         cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss") 
              << " adding: " << commands[2] << " as a player " 
              << wantevents << endl;
@@ -438,7 +449,7 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
 
         EncoderLink *enc = iter.data();
 
-        enc->SpawnReadThread(socket);
+        enc->SetReadThreadSock(socket);
 
         if (enc->isLocal())
         {
@@ -500,7 +511,7 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
     }
     else if (commands[1] == "FileTransfer")
     {
-        VERBOSE("MainServer::HandleAnnounce FileTransfer");
+        VERBOSE(VB_GENERAL, "MainServer::HandleAnnounce FileTransfer");
         cout << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
              << " adding: " << commands[2] << " as a remote file transfer"
              << endl;
@@ -761,6 +772,139 @@ static void *SpawnDelete(void *param)
     return NULL;
 }
 
+void MainServer::HandleStopRecording(QStringList &slist, PlaybackSock *pbs)
+{
+    ProgramInfo *pginfo = new ProgramInfo();
+    pginfo->FromStringList(slist, 1);
+
+    DoHandleStopRecording(pginfo, pbs);
+}
+
+void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
+{
+    QSocket *pbssock = NULL;
+    if (pbs)
+        pbssock = pbs->getSocket();
+
+    if (ismaster && pginfo->hostname != gContext->GetHostName())
+    {
+        PlaybackSock *slave = getSlaveByHostname(pginfo->hostname);
+
+        int num = -1;
+
+        if (slave)
+        {
+            num = slave->StopRecording(pginfo);
+
+            if (num > 0)
+                (*encoderList)[num]->StopRecording();
+
+            if (pbssock)
+            {
+                QStringList outputlist = "0";
+                SendResponse(pbssock, outputlist);
+            }
+ 
+            delete pginfo;
+            return;
+        }
+    }
+
+    int recnum = -1;
+
+    QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
+    for (; iter != encoderList->end(); ++iter)
+    {
+        EncoderLink *elink = iter.data();
+
+        if (elink->isLocal() && elink->MatchesRecording(pginfo))
+        {
+            recnum = iter.key();
+
+            elink->StopRecording();
+
+            while (elink->IsBusyRecording() ||
+                   elink->GetState() == kState_ChangingState)
+            {
+                usleep(100);
+            }
+        }
+    }
+
+    QString thequery;
+    QSqlQuery query;
+
+    dblock.lock();
+
+    MythContext::KickDatabase(m_db);
+
+    QString startts = pginfo->startts.toString("yyyyMMddhhmm");
+    startts += "00";
+    QString endts = pginfo->endts.toString("yyyyMMddhhmm");
+    endts += "00";
+    QDateTime now(QDateTime::currentDateTime());
+    QString newendts = now.toString("yyyyMMddhhmm");
+    newendts += "00";
+
+    // Set the recorded end time to the current time
+    // (we're stopping the recording so it'll never get to its originally 
+    // intended end time)
+    thequery = QString("UPDATE recorded SET starttime = %1, endtime = %2 "
+                       "WHERE chanid = %3 AND title = \"%4\" AND "
+                       "starttime = %5 AND endtime = %6;")
+                       .arg(startts).arg(newendts).arg(pginfo->chanid)
+                       .arg(pginfo->title.utf8()).arg(startts).arg(endts);
+
+    query = m_db->exec(thequery);
+
+    if (!query.isActive())
+    {
+        MythContext::DBError("Stop recording program update", query);
+    }
+
+    dblock.unlock();
+
+    // If we change the recording times we must also adjust the .nuv file's name
+    QString fileprefix = gContext->GetFilePrefix();
+    QString oldfilename = pginfo->GetRecordFilename(fileprefix);
+    pginfo->endts = now;
+    QString newfilename = pginfo->GetRecordFilename(fileprefix);
+    QFile checkFile(oldfilename);
+
+    if (checkFile.exists())
+    {
+        if (!QDir::root().rename(oldfilename, newfilename, TRUE))
+        {
+            VERBOSE(VB_IMPORTANT, QString("Could not rename: %1 to %2")
+                                         .arg(oldfilename).arg(newfilename));
+        }
+    }
+    else
+    {
+        VERBOSE(VB_IMPORTANT, QString("File: %1 does not exist.")
+                                     .arg(oldfilename));
+
+        if (pbssock)
+        {
+            QStringList outputlist;
+            outputlist << "BAD: Tried to rename a file that was in "
+                          "the database but wasn't on the disk.";
+            SendResponse(pbssock, outputlist);
+
+            delete pginfo;
+            return;
+        }
+    }
+
+    if (pbssock)
+    {
+        QStringList outputlist = QString::number(recnum);
+        SendResponse(pbssock, outputlist);
+    }
+
+    delete pginfo;
+}
+
 void MainServer::HandleDeleteRecording(QStringList &slist, PlaybackSock *pbs)
 {
     ProgramInfo *pginfo = new ProgramInfo();
@@ -904,17 +1048,16 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
     delete pginfo;
 }
 
-void MainServer::HandleQueryFreeSpace(PlaybackSock *pbs)
+void MainServer::getFreeSpace(int &totalspace, int &usedspace)
 {
-    QSocket *pbssock = pbs->getSocket();
-
     struct statfs statbuf;
-    int totalspace = -1, usedspace = -1;
     if (statfs(recordfileprefix.ascii(), &statbuf) == 0) 
     {
-        totalspace = statbuf.f_blocks / (1024*1024/statbuf.f_bsize);
-        usedspace = (statbuf.f_blocks - statbuf.f_bfree) / 
-                    (1024*1024/statbuf.f_bsize);
+        // total space available to user is total blocks - reserved blocks
+        totalspace = (statbuf.f_blocks - (statbuf.f_bfree - statbuf.f_bavail)) /
+                      (1024*1024/statbuf.f_bsize);
+        usedspace = (statbuf.f_blocks - statbuf.f_bavail) /
+                     (1024*1024/statbuf.f_bsize);
     }
 
     if (ismaster)
@@ -933,8 +1076,8 @@ void MainServer::HandleQueryFreeSpace(PlaybackSock *pbs)
         }
     }
 
-    QStringList strlist;
-    QString filename = gContext->GetSetting("RecordFilePrefix") + "/nfslockfile.lock";
+    QString filename = gContext->GetSetting("RecordFilePrefix") + 
+                                             "/nfslockfile.lock";
     QFile checkFile(filename);
 
     if (!ismaster)
@@ -945,7 +1088,17 @@ void MainServer::HandleQueryFreeSpace(PlaybackSock *pbs)
             usedspace = 0;
         }
     }
+}
 
+void MainServer::HandleQueryFreeSpace(PlaybackSock *pbs)
+{
+    QSocket *pbssock = pbs->getSocket();
+
+    int totalspace = -1, usedspace = -1;
+
+    getFreeSpace(totalspace, usedspace);
+
+    QStringList strlist;
     strlist << QString::number(totalspace) << QString::number(usedspace);
     SendResponse(pbssock, strlist);
 }
@@ -1110,9 +1263,9 @@ void MainServer::HandleLockTuner(PlaybackSock *pbs)
 
         if (retval != -1)
         {
-            QString msg = QString("Cardid %1 LOCKed for external use by '%2'.")
+            QString msg = QString("Cardid %1 LOCKed for external use on %2.")
                                   .arg(retval).arg(pbshost);
-            VERBOSE(msg);
+            VERBOSE(VB_GENERAL, msg);
 
             QString querystr = QString("SELECT videodevice, audiodevice, "
                                             "vbidevice "
@@ -1175,9 +1328,9 @@ void MainServer::HandleFreeTuner(int cardid, PlaybackSock *pbs)
         encoder = iter.data();
         encoder->FreeTuner();
         
-        QString msg = QString("Cardid %1 FREED from external use by '%2'.")
+        QString msg = QString("Cardid %1 FREED from external use on %2.")
                               .arg(cardid).arg(pbs->getHostname());
-        VERBOSE(msg);
+        VERBOSE(VB_GENERAL, msg);
 
         dblock.lock();
         ScheduledRecording::signalChange(m_db);
@@ -1268,7 +1421,9 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
     {
         cerr << QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss")
              << " Unknown encoder." << endl;
-        exit(0);
+        QStringList retlist = "bad";
+        SendResponse(pbssock, retlist);
+        return;
     }
 
     EncoderLink *enc = iter.data();  
@@ -1548,7 +1703,7 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
     }
     else if (command == "DONE_RINGBUF")
     {
-        enc->KillReadThread();
+        enc->SetReadThreadSock(NULL);
 
         retlist << "OK";
     }
@@ -1784,6 +1939,25 @@ void MainServer::HandleMessage(QStringList &slist, PlaybackSock *pbs)
     SendResponse(pbssock, retlist);
 }
 
+void MainServer::HandleIsRecording(QStringList &slist, PlaybackSock *pbs)
+{
+    (void)slist;
+
+    QSocket *pbssock = pbs->getSocket();
+    int RecordingsInProgress = 0;
+    QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
+    for (; iter != encoderList->end(); ++iter)
+    {
+        EncoderLink *elink = iter.data();
+        if (elink->IsBusyRecording())
+            RecordingsInProgress++;
+    }
+
+    QStringList retlist = QString::number(RecordingsInProgress);
+
+    SendResponse(pbssock, retlist);
+}
+
 void MainServer::HandleGenPreviewPixmap(QStringList &slist, PlaybackSock *pbs)
 {
     QSocket *pbssock = pbs->getSocket();
@@ -2123,39 +2297,163 @@ void MainServer::reconnectTimeout(void)
 void MainServer::PrintStatus(QSocket *socket)
 {
     QTextStream os(socket);
+    QString shortdateformat = gContext->GetSetting("ShortDateFormat", "M/d");
+    QString timeformat = gContext->GetSetting("TimeFormat", "h:mm AP");
+
     os.setEncoding(QTextStream::UnicodeUTF8);
 
+    QDateTime qdtNow = QDateTime::currentDateTime();
+
     os << "HTTP/1.0 200 Ok\r\n"
        << "Content-Type: text/html; charset=\"utf-8\"\r\n"
        << "\r\n";
 
     os << "<HTTP>\r\n"
        << "<HEAD>\r\n"
-       << "  <TITLE>MythTV Status</TITLE>\r\n"
+       << "  <TITLE>MythTV Status - " 
+       << qdtNow.toString(shortdateformat) 
+       << " " << qdtNow.toString(timeformat) << "</TITLE>\r\n"
        << "</HEAD>\r\n"
        << "<BODY>\r\n";
 
+    // encoder information ---------------------
     QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
     for (; iter != encoderList->end(); ++iter)
     {
         EncoderLink *elink = iter.data();
 
-        if (elink->isLocal())
+        bool bIsLocal = elink->isLocal();
+
+        if (bIsLocal)
         {
-            os << "Encoder: " << elink->getCardId() << " is local.\r\n";
+            os << "Encoder: " << elink->getCardId() << " is local ";
         }
         else
         {
-            os << "Encoder: " << elink->getCardId() << " is remote and ";
+            os << "Encoder: " << elink->getCardId() << " is remote ";
             if (elink->isConnected())
-                os << "is connected\r\n";
+                os << "(currently connected) ";
             else
-                os << "is not connected\r\n";
+                os << "(currently not connected) ";
         }
 
+        TVState encstate = elink->GetState();
+
+        if (encstate == kState_WatchingLiveTV)
+        {
+            os << "and is watching Live TV.\r\n";
+        }
+        else if (encstate == kState_RecordingOnly || 
+                 encstate == kState_WatchingRecording)
+        {
+            os << "and is recording";
+            if (bIsLocal)
+            {
+                ProgramInfo *pi = elink->GetRecording();
+                if (pi)
+                {
+                    os << " '" << pi->title << "'.  This recording will end "
+                       << "at " << (pi->endts).toString(timeformat);
+                    delete pi;
+                }
+            }
+            os << ".\r\n";
+        }
+        else
+            os << "and is not recording.\r\n";
+
         os << "<br>\r\n";
     }
 
+    // upcoming shows ---------------------
+    dblock.lock();
+    MythContext::KickDatabase(m_db);
+    Scheduler *sched = new Scheduler(false, encoderList, m_db);
+    sched->FillRecordLists(false);
+    list<ProgramInfo *> *recordinglist = sched->getAllPending();
+    dblock.unlock();
+
+    unsigned int iNum = 5;
+    if (recordinglist->size() < iNum) 
+    {
+        iNum = recordinglist->size();
+    }
+
+    if (iNum == 0)
+    {
+        os << "<P>There are no shows scheduled for recording.\r\n";
+    }
+    else
+    {
+       os << "<P>The next " << iNum << " show" << (iNum == 1 ? "" : "s" )
+          << " that " << (iNum == 1 ? "is" : "are") 
+          << " scheduled for recording:<BR>\r\n";
+
+       os << "<TABLE BORDER WIDTH=80%>\r\n"; 
+       os << "<TR><TH>Start Time</TH><TH>Show</TH><TH>Encoder</TH></TR>\r\n";
+       list<ProgramInfo *>::iterator iter = recordinglist->begin();
+       for (unsigned int i = 0; (iter != recordinglist->end()) && i < iNum; 
+            iter++, i++)
+       {
+           if (!(*iter)->recording ||     // bad entry, don't show as upcoming
+               ((*iter)->startts) < QDateTime::currentDateTime()) // recording
+           {                       
+               i--;
+           }
+           else
+           {
+               os << "<TR " << ((i % 2 == 0) ? "BGCOLOR=EEEEEE" : "") << ">" 
+                  << "<TD>" << ((*iter)->startts).toString(shortdateformat) 
+                  << " " << ((*iter)->startts).toString(timeformat) << "</TD>" 
+                  << "<TD>" << (*iter)->title << "</TD>"
+                  << "<TD>" << (*iter)->cardid << "</TD></TR>\r\n";
+           }
+       }
+       os << "</TABLE>";
+    }
+
+    delete sched;
+
+    os << "<P>Machine Information:\r\n";
+    os << "<TABLE WIDTH =100% BGCOLOR=EEEEEE>";
+    os << "<TR><TD>";
+
+    // drive space   ---------------------
+    int iTotal = -1, iUsed = -1;
+    QString rep;
+
+    getFreeSpace(iTotal, iUsed);
+
+    os << "Disk Usage:\r\n";
+
+    os << "<UL><LI>Total Space: ";
+    rep.sprintf(tr("%d,%03d MB "), (iTotal) / 1000, (iTotal) % 1000);
+    os << rep << "\r\n";
+
+    os << "<LI>Space Used: ";
+    rep.sprintf(tr("%d,%03d MB "), (iUsed) / 1000, (iUsed) % 1000);
+    os << rep << "\r\n";
+
+    os << "<LI>Space Free: ";
+    rep.sprintf(tr("%d,%03d MB "), (iTotal - iUsed) / 1000,
+                (iTotal - iUsed) % 1000);
+    os << rep << "</UL>\r\n";
+
+    os << "</TD><TD>";
+    
+    // load average ---------------------
+    double rgdAverages[3];
+    if (getloadavg(rgdAverages, 3) != -1)
+    {
+        os << "This machine's load average:";
+        os << "<UL><LI>1 Minute: " << rgdAverages[0] << "\r\n";
+        os << "<LI>5 Minute: " << rgdAverages[1] << "\r\n";
+        os << "<LI>15 Minute: " << rgdAverages[2] << "\r\n</UL>";
+    }
+
+    os << "</TD></TR></TABLE>";    // end the machine information table
+
     os << "</BODY>\r\n"
        << "</HTTP>\r\n";
 }
+
