@@ -50,6 +50,8 @@ class MainServer : public QObject
 
     // playback
     void HandleQueryRecordings(QString type, PlaybackSock *pbs);
+    void HandleStopRecording(QStringList &slist, PlaybackSock *pbs);
+    void DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs);
     void HandleDeleteRecording(QStringList &slist, PlaybackSock *pbs);
     void DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs);
     void HandleQueryFreeSpace(PlaybackSock *pbs);
@@ -67,6 +69,7 @@ class MainServer : public QObject
     void HandleGetRecorderFromNum(QStringList &slist, PlaybackSock *pbs);
     void HandleMessage(QStringList &slist, PlaybackSock *pbs);
     void HandleGenPreviewPixmap(QStringList &slist, PlaybackSock *pbs);
+    void HandleIsRecording(QStringList &slist, PlaybackSock *pbs);
     void HandleFillProgramInfo(QStringList &slist, PlaybackSock *pbs);
     void HandleQueueTranscode(QStringList &slist, PlaybackSock *pbs,
                               int state);
@@ -77,6 +80,8 @@ class MainServer : public QObject
 
     void SendResponse(QSocket *pbs, QStringList &commands);
 
+    void getFreeSpace(int &total, int &used);
+
     PlaybackSock *getSlaveByHostname(QString &hostname);
     PlaybackSock *getPlaybackBySock(QSocket *socket);
     FileTransfer *getFileTransferByID(int id);
