@@ -18,6 +18,7 @@ using namespace std;
 #include "libmythtv/scheduledrecording.h"
 #include "encoderlink.h"
 #include "libmyth/mythcontext.h"
+#include "remoteutil.h"
 
 Scheduler::Scheduler(bool runthread, QMap<int, EncoderLink *> *tvList, 
                      QSqlDatabase *ldb)
@@ -26,8 +27,6 @@ Scheduler::Scheduler(bool runthread, QMap<int, EncoderLink *> *tvList,
     db = ldb;
     m_tvList = tvList;
 
-    pthread_mutex_init(&schedulerLock, NULL);
-
     setupCards();
 
     threadrunning = runthread;
@@ -213,11 +212,29 @@ bool Scheduler::FillRecordLists(bool doautoconflicts)
         MarkConflicts();
     }
 
-    // PrintList();
+    if ((print_verbose_messages & VB_SCHEDULE) != 0)
+        PrintList();
 
     return hasconflicts;
 }
 
+void Scheduler::FillRecordListFromMaster(void)
+{
+    vector<ProgramInfo *> reclist;
+
+    RemoteGetAllPendingRecordings(reclist);
+
+    vector<ProgramInfo *>::iterator pgiter = reclist.begin();
+
+    for (; pgiter != reclist.end(); pgiter++)
+    {
+        ProgramInfo *pginfo = (*pgiter);
+        pginfo->schedulerid = pginfo->startts.toString() + "_" + pginfo->chanid;
+
+        recordingList.push_back(*pgiter);
+    }
+}
+
 void Scheduler::PrintList(void)
 {
     QString totrank;
@@ -903,7 +920,10 @@ list<ProgramInfo *> *Scheduler::CopyList(list<ProgramInfo *> *sourcelist)
 
                 if (!m_tvList->contains(second->cardid))
                 {
-                    cerr << "missing: " << second->cardid << " in tvList\n";
+                    if (m_tvList->size())
+                    {
+                        cerr << "Missing: " << second->cardid << " in tvList\n";
+                    }
                     continue;
                 }
 
@@ -1011,7 +1031,11 @@ void Scheduler::DoMultiCard(void)
 
                     if (!m_tvList->contains(lower->cardid))
                     {
-                        cerr << "Missing: " << lower->cardid << " in tvList\n";
+                        if (m_tvList->size())
+                        {
+                            cerr << "Missing: " << lower->cardid <<
+                                    " in tvList\n";
+                        }
                         continue;
                     }
 
@@ -1061,7 +1085,11 @@ void Scheduler::DoMultiCard(void)
 
                     if (!m_tvList->contains(higher->cardid))
                     {
-                        cerr << "Missing: " << higher->cardid << " in tvList\n";
+                        if (m_tvList->size())
+                        {
+                            cerr << "Missing: " << higher->cardid <<
+                                    " in tvList\n";
+                        }
                         continue;
                     }
 
@@ -1154,15 +1182,16 @@ void Scheduler::RunScheduler(void)
     {
         curtime = QDateTime::currentDateTime();
 
-        pthread_mutex_lock(&schedulerLock);
-
         if (CheckForChanges() ||
             (lastupdate.date().day() != curtime.date().day()))
         {
             FillEncoderFreeSpaceCache();
             FillRecordLists();
             lastupdate = curtime;
-            VERBOSE("Found changes in the todo list.");
+            VERBOSE(VB_GENERAL, "Found changes in the todo list.");
+
+            MythEvent me("SCHEDULE_CHANGE");
+            gContext->dispatch(me);
 
             // Determine if the user wants us to start recording early
             // and by how many seconds
@@ -1193,21 +1222,25 @@ void Scheduler::RunScheduler(void)
 
             if (recording && nexttv->isLowOnFreeSpace())
             {
-                QString msg = QString("SUPPRESSED recording \"%1\" on channel: "
-                                      "%2 on cardid: %3, sourceid %4.  Only "
-                                      "%5 Megs of disk space available.")
-                                      .arg(nextRecording->title)
-                                      .arg(nextRecording->chanid)
-                                      .arg(nextRecording->cardid)
-                                      .arg(nextRecording->sourceid)
-                                      .arg(nexttv->getFreeSpace());
+                FillEncoderFreeSpaceCache();
+                if (nexttv->isLowOnFreeSpace())
+                {
+                    QString msg = QString("SUPPRESSED recording '%1' on channel"
+                                          " %2 on cardid %3, sourceid %4.  Only"
+                                          " %5 Megs of disk space available.")
+                                          .arg(nextRecording->title)
+                                          .arg(nextRecording->chanid)
+                                          .arg(nextRecording->cardid)
+                                          .arg(nextRecording->sourceid)
+                                          .arg(nexttv->getFreeSpace());
 
-                VERBOSE(msg);
+                    VERBOSE(VB_GENERAL, msg);
 
-                RemoveRecording(nextRecording);
-                nextRecording = NULL;
-                recIter = recordingList.begin();
-                continue;
+                    RemoveRecording(nextRecording);
+                    nextRecording = NULL;
+                    recIter = recordingList.begin();
+                    continue;
+                }
             }
 
             if (recording && nexttv->isTunerLocked())
@@ -1220,7 +1253,7 @@ void Scheduler::RunScheduler(void)
                                       .arg(nextRecording->cardid)
                                       .arg(nextRecording->sourceid);
 
-                VERBOSE(msg);
+                VERBOSE(VB_GENERAL, msg);
 
                 RemoveRecording(nextRecording);
                 nextRecording = NULL;
@@ -1235,7 +1268,7 @@ void Scheduler::RunScheduler(void)
                 secsleft -= prerollseconds;
             }
 
-//            VERBOSE(secsleft << " seconds until " << nextRecording->title);
+//            VERBOSE(VB_GENERAL, secsleft << " seconds until " << nextRecording->title);
 
             if (secsleft > 30)
                 break;
@@ -1268,7 +1301,7 @@ void Scheduler::RunScheduler(void)
                                              .arg(nextRecording->cardid)
                                              .arg(nextRecording->sourceid);
 
-                VERBOSE(msg);
+                VERBOSE(VB_GENERAL, msg);
 
                 AddToDontRecord(nextRecording);
                 RemoveRecording(nextRecording);
@@ -1292,8 +1325,6 @@ void Scheduler::RunScheduler(void)
                 recIter++;
         }
 
-        pthread_mutex_unlock(&schedulerLock);
-
         sleep(1);
     }
 } 
