@@ -23,6 +23,7 @@ class Scheduler : public QObject
 
     bool CheckForChanges(void);
     bool FillRecordLists(bool doautoconflicts = true);
+    void FillRecordListFromMaster(void);
 
     void FillEncoderFreeSpaceCache();
 
@@ -95,8 +96,6 @@ class Scheduler : public QObject
 
     QMap<QString, bool> recPendingList;
 
-    pthread_mutex_t schedulerLock;
-
     bool threadrunning;
 
     void AddToDontRecord(ProgramInfo *pginfo);
