@@ -45,15 +45,26 @@ class PlaybackBox : public MythDialog
     void pageUp() { cursorUp(true); }
     void selected();
     void playSelected();
+    void stopSelected();
     void deleteSelected();
     void expireSelected();
+    void showActionsSelected();
 
+    void doPlay();
+
+    void askStop();
+    void doStop();
+    void noStop();
+
+    void askDelete();
     void doDelete();
     void noDelete();
 
     void doAutoExpire();
     void noAutoExpire();
 
+    void doCancel();
+
     void exitWin();
 
   protected:
@@ -61,19 +72,22 @@ class PlaybackBox : public MythDialog
     void keyPressEvent(QKeyEvent *e);
 
   private:
-    void FillList(void);
+    bool FillList(void);
     void UpdateProgressBar(void);
 
     QString cutDown(QString, QFont *, int);
     QPixmap getPixmap(ProgramInfo *);
     QPainter backup;
     void play(ProgramInfo *);
+    void stop(ProgramInfo *);
     void remove(ProgramInfo *);
     void expire(ProgramInfo *);
+    void showActions(ProgramInfo *);
 
     bool skipUpdate;
     bool noUpdate;
     bool pageDowner;
+    bool connected;
     ProgramInfo *curitem;
     ProgramInfo *delitem;
 
@@ -104,7 +118,11 @@ class PlaybackBox : public MythDialog
 
     void doRemove(ProgramInfo *);
     void promptEndOfRecording(ProgramInfo *);
-    void showDeletePopup(int);
+    void showDeletePopup(ProgramInfo *, int);
+    void showActionPopup(ProgramInfo *program);
+    void initPopup(MythPopupBox *popup, ProgramInfo *program, 
+                   QString message, QString message2);
+    void cancelPopup();
 
     bool fileExists(ProgramInfo *pginfo);
 
