@@ -407,6 +407,7 @@ void ViewScheduled::updateList(QPainter *p)
     QString tempDate = "";
     QString tempTime = "";
     QString tempChan = "";
+    QString tempCard = "";
 
     ConflictData::Iterator it;
     ConflictData::Iterator start;
@@ -447,6 +448,11 @@ void ViewScheduled::updateList(QPainter *p)
                         else
                             tempChan = tempInfo->chanstr;
 
+                        if (tempInfo->recording)
+                            tempCard = QString::number(tempInfo->cardid);
+                        else
+                            tempCard = "-";
+
                         if (cnt == inList)
                         {
                             if (curitem)
@@ -458,6 +464,7 @@ void ViewScheduled::updateList(QPainter *p)
                         ltype->SetItemText(cnt, 1, tempChan);
                         ltype->SetItemText(cnt, 2, tempDate + " " + tempTime);
                         ltype->SetItemText(cnt, 3, tempSubTitle);
+                        ltype->SetItemText(cnt, 4, tempCard);
 
                         if (tempInfo->conflicting)
                         {
@@ -640,7 +647,7 @@ void ViewScheduled::updateInfo(QPainter *p)
 
 void ViewScheduled::edit()
 {
-    if (doingSel)
+    if (doingSel || !curitem)
         return;
 
     doingSel = true;
@@ -649,33 +656,30 @@ void ViewScheduled::edit()
 
     MythContext::KickDatabase(db);
 
-    if (rec)
+    if ((gContext->GetNumSetting("AdvancedRecord", 0)) ||
+        (rec->GetProgramRecordingStatus(db) > kAllRecord))
     {
-        if ((gContext->GetNumSetting("AdvancedRecord", 0)) ||
-            (rec->GetProgramRecordingStatus(db) > kAllRecord))
-        {
-            ScheduledRecording record;
-            record.loadByProgram(db, rec);
-            record.exec(db);
-        }
-        else
-        {
-            InfoDialog diag(rec, gContext->GetMainWindow(), "Program Info");
-            diag.exec();
-        }
-
-        ScheduledRecording::signalChange(db);
-
-        FillList();
-        update(fullRect);
+        ScheduledRecording record;
+        record.loadByProgram(db, rec);
+        record.exec(db);
     }
+    else
+    {
+        InfoDialog diag(rec, gContext->GetMainWindow(), "Program Info");
+        diag.exec();
+    }
+
+    ScheduledRecording::signalChange(db);
+
+    FillList();
+    update(fullRect);
 
     doingSel = false;
 }
 
 void ViewScheduled::selected()
 {
-    if (doingSel)
+    if (doingSel || !curitem)
         return;
 
     doingSel = true;
@@ -700,7 +704,7 @@ void ViewScheduled::selected()
     {
         handleConflicting(rec);
     }
-    else if (rec)
+    else
     {
         if ((gContext->GetNumSetting("AdvancedRecord", 0)) ||
             (rec->GetProgramRecordingStatus(db) > kAllRecord))
@@ -762,6 +766,9 @@ void ViewScheduled::handleNotRecording(ProgramInfo *rec)
     vector<ProgramInfo *> *conflictlist = RemoteGetConflictList(rec, false);
     allowKeys = true;
 
+    if (!conflictlist)
+        return;
+
     QString dstart, dend;
     vector<ProgramInfo *>::iterator i;
     for (i = conflictlist->begin(); i != conflictlist->end(); i++)
@@ -840,6 +847,9 @@ void ViewScheduled::chooseConflictingProgram(ProgramInfo *rec)
     vector<ProgramInfo *> *conflictlist = RemoteGetConflictList(rec, true);
     allowKeys = true;
 
+    if (!conflictlist)
+        return;
+
     QString message = tr("The follow scheduled recordings conflict with each "
                          "other.  Which would you like to record?");
 
