@@ -40,12 +40,15 @@ int main(int argc, char *argv[])
         return -1;
     }
 
-    QString server = gContext->GetSetting("MasterServerIP", "localhost");
-    int port = gContext->GetNumSetting("MasterServerPort", 6543);
-    gContext->ConnectServer(server, port);
-
     gContext->LoadQtConfig();
 
+    QString auddevice = gContext->GetSetting("AudioOutputDevice");
+    if (auddevice == "" || auddevice == QString::null)
+    {
+        cerr << "You need to run 'mythfrontend', not 'mythtv'.\n";
+        exit(-1);
+    }
+
     MythMainWindow *mainWindow = new MythMainWindow();
     mainWindow->Show();
     gContext->SetMainWindow(mainWindow);
